<?php
/**
 * Programa         : ParserWrapper
 * Objetivo         : Parse documento HTML
 * Programador      : Alexandre P. Trindade
                         Baseado no nucleo da classe parseWrapper escrita por
                             - Claudio h. Hirose
                             - Joo Gustavo Ferreira Farias
			                 - Edson Carlos da Silva

 * Data Incio      : 30 de setembro de 2002 - segunda-feira
 * Ultima alteracao : 19 de Maio de 2011 - Quinta-feira   (Alexandre)
 * Verso           : U 1.6.3

   Histrico da Classe ParserWrapper

      O conceito foi baseado na classe ParserWrapper original (autores citados acima) que continha
      apenas o metodo makeRepeat e operava sobre um unico arquivo de template.

      Apartir de Maro de 2004 o cdigo original foi modificado e expandido e integrado 
      biblioteca uLib (aut. Alexandre P. Trindade) tornando-se mais
      abrangente flexivel de modo a suportar o desenvolvimento em separado das templates e
      os scripts PHP.
 **/

/*
   =======================================================================================
   Verso 1.6.3 - Alteraoes   (Alexandre) - 13/05/2011

   Adicionado Funo:  makeSelect
                    - Implementado o prenchimento dos items de um combobox (tag SELECT no
                      Html) definidos por array simples ou apartir do resultado de uma
                      consulta uMySQLQuery.
      Forma: <exec name=makeSelect arr=array_select></exec>

      Para montar array_select (makeSelect), utilizar o metodo addSelectValue(varArray,"Label","Value");
        exemplo makeSelect (PHP):
               $pw  = new ParserWrapper();
               $pw->addSelectValue($a_select,"Opcao 1", "1");
               $pw->addSelectValue($a_select,"Opcao 2", "2");
               $pw->addSelectValue($a_select,"Opcao 3", "3",true); // este entrar como item selecionado

      Para makeSelect apartir de banco de dados, utilizar o mtodo
        getArraySelect("field_label","field_value") da classe uMySQLQuery.
        exemplo makeSelect com uMySQLQuery(PHP)
           $QRY = new uMySQLQuery();
           ... seta conexo
           $QRY->SQLAdd("SELECT ID,NOME,UF");
           $QRY->SQLAdd(" FROM TAB_ESTADOS ORDER BY NOME");
           $QRY->Open();
           $a_select = $QRY->getArraySelect("NOME","UF","ID",10); //Obtem o array de preenchomento e marca o registro com ID=10 como selecionado

          Obs: Para determinar qual ser o item selecionado usar o evento onLoadRec e definir o campo selected="selected".
      Finalidade simplificar este trabalho.
   =======================================================================================
   Verso 1.6.2 - Alteraoes   (Alexandre)

   Alterado Funo: parseFile()
                    - Adaptado para suportar urls como nome de arquivo

      Finalidade permitir o uso de layouts remotos (provenientes de sites diferentes)
   =======================================================================================
   Verso 1.6.1 - Alteraoes       (Alexandre)

   Adicionado a funo de bloco "doIfExp".
      Forma: <exec name=doIfExp var=nome_variavel_booleana>

      Finalidade avaliar macros nas tags.
   =======================================================================================
   Verso 1.6.0 - Alteraoes       (Alexandre)

   Adicionado a funo de bloco "doIf" e "doIfNot".
      Forma: <exec name=doIf var=nome_variavel_booleana>

   =======================================================================================
   Verso 1.5.2 - Alteraes       (Alexandre)
   -------------------------

   Adicionada a capacidade de utilizao de variveis para definio de tags.
   Esta capacidade permite que um mesmo template possa ser combinado dinmicamente
   com outros, e que tags de blocos possam ser definidas de acordo com o resultado
   de um processo pelo cdio PHP.

   Mtodo: addReplaceVars($alista,$pref,$suf)
   Parametros:
       $alista - Array contendo a lista de tags a serem substituidas
	             o ndice do array corresponde ao nome da variavel no arquivo,
				 e o seu valor ser utilizado na substituio.
	   $pref   - Prefixo de identificao de variavel (p.ex: "<%!")
	   $suf    - Sufixo de identificao de varivel  (p.ex: "!%>")

	   Nota: o nome da variavel no arquivo deve ser composta por:
	         prefixo + nome + sufixo         (exemplo: <%!nome_arquivo!%>)
*/

global $parse_ver;
global $parse_data;

$parse_ver  = "1.6.3";
$parse_data = "13/05/2011";

define('PARSING_DEFAULT_DELIMITER', '|');
define('PARSING_ISVAR', 1);
define('PARSING_ISCONST', 2);
define('PARSING_KEY', 3);
define('SELECTTAG','<option value="<%value%>" <%selected%>><%option%></option>'."\r\n");

//r: mantem legibilidade do layout o Dreamweever

function NoPHPTag($a_linhas){
	return str_replace(array('<?php','?>'),array('<!--','-->'),$a_linhas);
}


function safe_array_merge(){
   $result = array();
   $count = func_num_args();
   $i = 0;
   while ($i<$count){
      $v = func_get_arg($i);
      if (is_array($v)){
         $result = array_merge($result, $v);
      }else if (!empty($v) ) {
         array_push($result, $v);
      }
      $i++;
   }
   return $result;
}

class ParserWrapper {
		
		var $param = array();
		var $vari     = '<%';
		var $varf     = '%>';
		var $arqi     = '<%%';
		var $arqf     = '%%>';
		var $varnum   = 2;
		var $arqnum   = 2;
		var $tagnome  = '[eE][xX][eE][cC]';
		var $tagorig  = 'exec';
		var $varnome  = '[a-zA-Z_][a-zA-Z0-9._]*';
		var $tagmeio  = '.*';
		var $arqnome  = '(\.\/|\.\.\/|[\/]?)[a-zA-Z0-9_][a-zA-Z0-9_\/\.]*';
		var $funcnome = '[a-zA-Z_][a-zA-Z0-9_]*';
		var $basicstand = array(
							'parseStr', 'parseFile', 'parseTag',
				  			'parseStrTag', 'parseTagStr', 'parseFileTag', 'parseTagFile',
				  			'parseStrTagFile', 'parseTagStrFile',
				  			'parseFileTagStr', 'parseFileStrTag',
				  			'parseStrFileTag', 'parseTagFileStr');
	   var $stand    = array('makeSelect','doIf','doIfNot','doIfExp','makeBool', 'makeArr', 'makeRepeat','noRepeat', 'fileInclude', 'makeChain');
	   var $htmls    = array('makeSelect');
	   var $maxreg   = 1000;
	   var $colors   = array();
	   var $swapvars = array(array("<%!","!%>"),array());
	   var $extendedTags = array("makeselect" =>SELECTTAG);

       /**
        *  Adiciona opo de select (comboBox html).
        *  @param <var array>   $aSelect   - Array no qual as opes do select sero incluidas
        *  @param <string>      $option    - String contendo o texto (opo) a ser selecionada pelo usurio
        *  @param <string>      $value     - Valor retornado quando o usurio seleciona esta opo
        *  @param <bool>        $selected  - Indica se esta opo estar selecionado inicialmente
        */
       function addSelectValue(&$aSelect,$option,$value,$selected=false){
          if (!is_array($aSelect)){ return false;}
          if ($selected == true){
             $i = 0; $j = count($aSelect);
             while ($i < $j){
               $p = $aSelect[$i]["selected"];
               $aSelect[$i]["selected"] = "";
               $i += 1;
             }
          }

          $s = ($selected)? "selected":"";
          $a = array("option"=>$option, "value"=>$value, "selected"=>$s);
          array_push($aSelect,$a);
       }

       function addReplaceVars($alista, $pi, $pf){
 	      $this->swapvars = array();
		  if (count($alista) > 0){
			  $this->swapvars[0] = array($pi,$pf);
			  $this->swapvars[1] = $alista;
		  }
	   }
	   function setMax($maxreg) {
	            $this->maxreg = $maxreg;
       }
       function addColors($colors) {
		       $this->colors = safe_array_merge($this->colors, $colors );
       }

		function makeValid($str = ''){
			return ($str ? str_replace('/', '\/', preg_quote($str)) : '');
		}


		function extractFromComment($str = ''){
			if ( (!$str) || (!($str = trim($str))) ) return '';
			if ( ($pos1 = strpos($str, '<!--')) === false)  return $str;
			$pos1 += 4;
			if ( ($pos2 = strpos($str, '-->', $pos1)) === false) return substr($str, $pos1);
			return substr($str, $pos1, $pos2-$pos1);
		}

		function get1stCommentedValue(&$str, $before = '<!--', $after = '-->', $blen = 4, $alen = 3){
			if ( (!$before) || (!$after) ) return '';
			if ( ($pos = strpos($str, $after)) === false) return '';
			$res = substr($str, $blen, $pos - $blen); /* $blen = strlen('<!--') */
			$str = substr($str, $pos + $alen); /* $alen = strlen('-->') */
			return $res;
		}

		function makeColspan($str){ return preg_replace('/colspan\=(\")?(0|1)(\")?/is', '', $str); }		
		
		function setTag($nome, $valor) {
			$this->param[strtolower($nome)] = $valor;
		}
		
		function getTag($nome, $valor) {
			$this->param[strtolower($nome)] = $valor;
		}
		
		function parseValue($str, $parname = ''){
			if (!$str) return '';
			if (!$parname) 
				return (($pos = strpos($str, ' ')) === false ? $str : substr($str, 0, $pos));
			if ( ($pos = strpos(strtolower($str), $parname.'=')) === false ) return '';
			$str = substr($str, $pos+strlen($parname.'='));
			return ( ($pos = strpos($str, ' ')) === false ? $str : substr($str, 0, $pos));
		}
	
		function parseStr($str = '', $v = ''){

			if (!$str) return '';
			if (is_object($v)) 	
			   $v = get_object_vars($v);
			else 
			if (!is_array($v)) 
			   $v = array();

         $p2  = '(isset($v["\\'.$this->varnum.'"])? $v["\\'.$this->varnum.'"]: (isset($v[strtoupper("\\';
         $p2 .= $this->varnum.'")]) ? $v[strtoupper("\\'.$this->varnum.'")]: (  isset($GLOBALS["\\';
         $p2 .= $this->varnum.'"])? $GLOBALS["\\'.$this->varnum.'"] : (defined("\\'.$this->varnum.'")? constant("\\';
         $p2 .= $this->varnum.'"): ""   )))); ';
			$str = preg_replace('/('.$this->vari.')('.$this->varnome.')('.$this->varf.')/e',$p2,$str);

			return $str;
		}


      //r: Centraliza processos de carga do template
      function PW_Preload($linhas){
			$linhas = NoPHPTag($this->PW_VarReplace($linhas));
		    return $linhas;
		}

		function parseFile($arqnome = '', $v = '')
		{
			if (!$arqnome) return '';
			//r: verifica se o link nao  uma url
			if (strpos($arqnome,"://") === false && !@file_exists($arqnome)) {
                $arqnome = $_SERVER["DOCUMENT_ROOT"]."/_modelo_/".$arqnome;
                if (!@file_exists($arqnome)) {
                        return (isset($GLOBALS[$arqnome]) ? $GLOBALS[$arqnome] : '');
                }
         }
		
			return preg_replace('/('.$this->arqi.')('.$this->arqnome.')('.$this->arqf.')/e',
					    '$this->parseFile("\\'.$this->arqnum.'", $v)',
					    join('', $this->PW_Preload(file($arqnome))));
		}
		
		function parseTag($str = '', $v = '')
		{
            if ( !$str ){return $str;}
            
			$str = preg_replace (
				'/<'.$this->tagnome.'([^>]*?) [Nn][Aa][Mm][Ee]=('.$this->funcnome.')([^>]*?)>('.$this->tagmeio.')<\/'.$this->tagnome.'>/se',
				'$this->getInner("\\4", "\\2", $v, "\\1 \\3")', 
				$str);

			return $str;
		}

		
		function getInner($str = '', $func = '', $v = '', $extra = '')
		{
            if (!in_array($func,$this->htmls)){
               if ( !($str = trim($str)) ){return $str;}
            }

			$ltag = strtolower($this->tagorig);
			$lstr = strtolower($str);

			if ( ($pos = strpos($lstr,  $ltag)) === false ||
			     ($pos1 = strpos($lstr, '<'.$ltag)) === false )
			{
				return ( $func && method_exists($this, $func)
					? ( in_array($func, $this->basicstand)
						? $this->$func($str, $v) 
						: ( in_array($func, $this->stand) 
					   		? $this->$func($str, $v, $extra) 
					   		: $this->$func($str) ) )
				   	:  $str );
			}
		
			$taglen = strlen('</'.$ltag.'>');
			$lstr .= '</'.$ltag.'>';
			
			$rpos2 = strrpos($lstr, '</'.$ltag.'>');
			$pos2 = strpos($lstr, '</'.$ltag.'>');
			
			while ( ($pos1 !== false) && ($pos2 !== false) && ($rpos2 !== false) &&
				($pos2 > $pos1) && ($pos2+$taglen < $rpos2) )
			{
				if ($pos1 !== false) $pos1 += $taglen;
				if ($pos2 !== false) $pos2 += $taglen+1;
				$pos1 = strpos($lstr, '<'.$ltag, $pos1);
				$pos2 = strpos($lstr, '</'.$ltag.'>', $pos2);
			}
			$str1 = substr($str, 0, $pos2);
			$str2 = substr($str, $pos2+$taglen);

			if ($str2)	$pos1 = strpos(strtolower($str2), '<'.$ltag);
			
			if ( $pos1 === false ) 	
			   $middle = '';
			else{
				$middle = substr($str2, 0, $pos1);
				$str2 = substr($str2, $pos1);
			}
			if ( !trim($str2) ){
			/*	if ( $pos1 !== false && $pos1 > $pos2 )
				   $str1 .= '</'.$ltag.'>';
				else 
				if ( !trim($middle) )
				   $str1 .= '</'.$ltag.'>';   */
			}
			$z=	( $func && method_exists($this, $func)
					? ( in_array($func, $this->basicstand)
						? $this->$func($str1, $v) 
						: ( in_array($func, $this->stand) 
					   		? $this->$func($str1, $v, $extra) 
					   		: $this->$func($str1) ) )
				   	:  $str1 ).
				$middle.
				( $str2 ? $this->parseTag($str2.'</'.$ltag.'>', $v) : $str2);
			return $z;
		}
		
		
		/******************* another helpful functions ************************/
		function parseStrTag($str = '', $v = '')
		{  return $this->parseStr( $this->parseTag($str, $v), $v);  }
		
		function parseTagStr($str = '', $v = '')
		{  return $this->parseTag( $this->parseStr($str, $v), $v); }
		
		function parseFileTag($str = '', $v = '')
		{  return $this->parseFile( $this->parseTag($str, $v), $v);  }
		
		function parseTagFile($fname = '', $v = '')
		{  return $this->parseTag( $this->parseFile($fname, $v), $v);  }
		
		function parseStrFile($fname = '', $v = '')
		{  return $this->parseStr( $this->parseFile($fname, $v), $v);  }
		
		function parseStrTagFile($fname = '', $v = '')
		{ return $this->parseStr($this->parseTag($this->parseFile($fname, $v), $v), $v); }
		
		function parseTagStrFile($fname = '', $v = '')
		{ return $this->parseTag($this->parseStr($this->parseFile($fname, $v), $v), $v); }
		
		function parse($fname = '', $v = '')
		{ return $this->parseStr($this->parseTag($this->parseFile($fname, $v), $v), $v); }
		

		/************** rare used helpful functions - just added for completeness **************/
		function parseFileStr($str = '', $v = '')
		{  return $this->parseFile( $this->parseStr($str, $v), $v);  }
		
		function parseFileTagStr($str = '', $v = '')
		{ return $this->parseFile($this->parseTag($this->parseStr($str, $v), $v), $v); }
		
		function parseFileStrTag($str = '', $v = '')
		{  return $this->parseFile($this->parseStr(parseTag($str, $v), $v), $v); }
		
		function parseStrFileTag($str = '', $v = '')
		{  return $this->parseStr($this->parseFile($this->parseTag($str, $v), $v), $v); }
		
		function parseTagFileStr($str = '', $v = '')
		{ return $this->parseTag($this->parseFile($this->parseStr($str, $v), $v), $v); }

		/***************************************************************************************/
		/***
		 *
		 *
		 **/

		function fileInclude($str = '', $v = '', $extra = '')
		{

			if (!trim($str)) return '';
			
			$parsearr = array('save' => PARSING_ISCONST, 
					  'func' => PARSING_ISCONST, 'ffunc'=>PARSING_ISCONST);
		
			foreach ($parsearr as $name => $parsetype){
				$tt = $this->parseValue($extra, $name);
				if (!strlen($tt)) continue;
		
				if ($parsetype != PARSING_ISVAR){ $$name = $tt; continue; }
		
				if ( isset($v[$tt])) { $$name = $v[$tt]; continue; }
				else if ( isset($GLOBALS[$tt]) )  $$name = &$GLOBALS[$tt];
				else $$name = '';
			}
		
			$fname = ( $ffunc ? ( in_array($ffunc, $this->basicstand)
							? $this->$ffunc($str, $v) 
							: ( in_array($ffunc, $this->stand) 
						   		? $this->$ffunc($str, $v, $extra) 
						   		: $this->$ffunc($str) ) )
						: $str );
						
			if ( (!trim($fname)) || (!file_exists($fname)) ) return '';
			
			$str = join('',  $this->PW_Preload(file($arqnome)));
			
			$res = ( $func ? ( in_array($func, $this->basicstand)
							? $this->$func($str, $v) 
							: ( in_array($func, $this->stand) 
						   		? $this->$func($str, $v, $extra) 
						   		: $this->$func($str) ) )
						: $str );
			if ( !empty($save) ) $GLOBALS[$save] = $res;
			return $res;
		}


		function makeBool($str = '', $v = '', $extra = '')
		{

			$parsearr = array('var'=>PARSING_ISVAR, 
					  'save'=>PARSING_ISCONST, 'func'=>PARSING_ISCONST, 
					  'type'=>PARSING_ISCONST, 'delim'=>PARSING_ISCONST);
			
			foreach ($parsearr as $name => $parsetype){
				$tt = $this->parseValue($extra, $name);
				if (!strlen($tt)) continue;
				if ($parsetype != PARSING_ISVAR){ $$name = $tt; continue; }
		
				if ( isset($v[$tt])) { $$name = $v[$tt]; continue; }
				else if ( isset($GLOBALS[$tt]) )  $$name = &$GLOBALS[$tt];
				else $$name = '';
			}
			
			if ( empty($var) ) $var = '';
			
			if ( is_object($var) )  $var = get_object_vars($var);
		
			$isEmpty = ( is_array($var) 
					? ( sizeof($var) ? false : true ) 
					: ( empty($var) ? true : false ) );
		
			if (empty($func) || is_object($func) || is_array($func) || (!method_exists($this, $func))) 
				$func = '';
			
			$isInverted = ( (!empty($type)) && $type == 'invert' ? true : false );
		
			$delim = ( empty($delim) ? PARSING_DEFAULT_DELIMITER : $delim );
			
			$true = $false = '';
			if ( $isInverted ) $false = $str;
			else @list($true, $false) = explode($delim, $str);
				
			if ( $isInverted xor $isEmpty ) $return = &$true;
			else				$return = &$false;
		
			$res = '';
			if ($func) {
				if ( in_array($func, $this->basicstand) )
					$res = $this->$func($return, $v) ;
				else  	$res = 
					 ( in_array($func, $this->stand) 
				   		? $this->func($return, $v, $extra) 
				   		: $this->func($return) ) ;
			}
			else $res = $return;
					
			if ( !empty($save) ) $GLOBALS[$save] = $res;
			return $res;
		}
		
		function makeArr($str = '', $v = '', $extra = '')
		{
			$parsearr = array('key' => PARSING_ISVAR, 	
					  'save' => PARSING_ISCONST, 
					  'func' => PARSING_ISCONST, 'delim' => PARSING_ISCONST);
		
			foreach ($parsearr as $name => $parsetype){
				$tt = $this->parseValue($extra, $name);
				if (!strlen($tt)) continue;
		
				if ($parsetype != PARSING_ISVAR){ $$name = $tt; continue; }
		
				if ( isset($v[$tt])) { $$name = $v[$tt]; continue; }
				else if ( isset($GLOBALS[$tt]) )  $$name = &$GLOBALS[$tt];
				else $$name = '';
			}
		
			if ( empty($key) || is_object($key) || is_array($key) || (!$key) ){
				$key = 0;
			}
			
			if (empty($func) || is_object($func) || is_array($func) || (!method_exists($this, $func))) 
				$func = '';
			
			$delim = ( empty($delim) ? PARSING_DEFAULT_DELIMITER : $delim );
			
			$arr = explode($delim, $str);
			if ( !sizeof($arr) ){
				if (!empty($save)) $GLOBALS[$save] = '';
				 return '';
			}
			
			$res = '';
			for ($i = 0; $i < sizeof($arr); $i++){
				if ($i != $key) continue;
				$res = ( $func ? ( in_array($func, $this->basicstand)
							? $this->$func($arr[$i], $v) 
							: ( in_array($func, $this->stand) 
						   		? $this->$func($arr[$i], $v, $extra) 
						   		: $this->$func($arr[$i]) ) )
						: $arr[$i] );
				break;
			}
			if ( !empty($save) ) $GLOBALS[$save] = $res;
			return $res;
		}
		
        function makeSelect($str = '', $v = '', $extra = ''){
           return $this->makeRepeat($this->extendedTags["makeselect"], $v, $extra);
        }

		function makeRepeat($str = '', $v = '', $extra = ''){
		
			$parsearr = array('arr' => PARSING_ISVAR,
					  'save' => PARSING_ISCONST, 'func' => PARSING_ISCONST);

			foreach ($parsearr as $name => $parsetype){
				$tt = $this->parseValue($extra, $name);
				if (!strlen($tt)) continue;
				if ($parsetype != PARSING_ISVAR){ $$name = $tt; continue; }

				if (is_array($v) &&  isset($v[$tt]) && isset($name) ) { $$name = $v[$tt]; continue; }
				else if ( isset($GLOBALS[$tt]) )  $$name = &$GLOBALS[$tt];
				else $$name = '';
			}
			
			if ( empty($arr) ) $arr = array();
			if ( is_object($arr) ) $arr = get_object_vars($arr);
			else if (!is_array($arr)) $arr = array($arr);

			$size = sizeof($arr);
			
			if (!$size) {
				if (!empty($save)) $GLOBALS[$save] = '';
				return '';
			}
			
			if (empty($func) || is_object($func) || is_array($func) || (!method_exists($this, $func))) {
				//r:$func = '';
				$func = 'parseStrTag';
			}

			$res = '';
			$cont = 0;
			$a_parsewraper_reg = array();
			foreach ($arr as $arrv){
				if (!empty($this->colors)) {
    				$GLOBALS["_color"] = current($this->colors);
				   if (!next($this->colors)) reset($this->colors);
            }
            $a_parsewraper_reg = $arrv;
            
			$temp = safe_array_merge($v, $arrv );
			
			$res .= ( $func ? ( in_array($func, $this->basicstand)
							? $this->$func($str, $temp, $arrv)
							: ( in_array($func, $this->stand) 
						   		? $this->$func($str, $temp, $extra, $a_parsewraper_reg)
						   		: $this->$func($arrv) ) )
						: $str );
				if ($cont++ > $this->maxreg) break;
			}
			if ( !empty($save) ) $GLOBALS[$save] = $res;

            while (strpos($res, "\'")>-1)
                $res = str_replace("\'", "'", $res);
			return $res;
		}
   
		function noRepeat($str = '', $v = '', $extra = '')
		{
		
			//r:print $extra."<BR>\n";
			$parsearr = array('arr' => PARSING_ISVAR, 
					  'save' => PARSING_ISCONST, 'func' => PARSING_ISCONST);
		
			foreach ($parsearr as $name => $parsetype){
				$tt = $this->parseValue($extra, $name);
				if (!strlen($tt)) continue;
				if ($parsetype != PARSING_ISVAR){ $$name = $tt; continue; }

				if (is_array($v) &&  isset($v[$tt]) && isset($name) ) { $$name = $v[$tt]; continue; }
				else if ( isset($GLOBALS[$tt]) )  $$name = &$GLOBALS[$tt];
				else $$name = '';
			}
			if ( empty($arr) ) $arr = array();
			if ( is_object($arr) ) $arr = get_object_vars($arr);
			else if (!is_array($arr)) $arr = array($arr);

			$size = sizeof($arr);
			
			if (!$size) {
				if (!empty($save)) $GLOBALS[$save] = '';
				return '';
			}
			
			if (empty($func) || is_object($func) || is_array($func) || (!method_exists($this, $func))) {
				//r:$func = '';
				$func = 'parseStrTag';
			}
		
			$res = '';
			$cont = 0;
			foreach ($arr as $arrv){
				if (!empty($this->colors)) {
    				$GLOBALS["_color"] = current($this->colors);
				   if (!next($this->colors))
				      reset($this->colors);
                }
                
	            $temp = safe_array_merge($v, $arrv );

				$res .= ( $func ? ( in_array($func, $this->basicstand)
							? $this->$func($str, $temp) 
							: ( in_array($func, $this->stand) 
						   		? $this->$func($str, $temp, $extra) 
						   		: $this->$func($arrv) ) )
						: $str );
				//r: if ($cont++ > $this->maxreg) break;
				break;
			}
			if ( !empty($save) ) $GLOBALS[$save] = $res;

            while (strpos($res, "\'")>-1)
                $res = str_replace("\'", "'", $res);
			return $res;
		}

		function doIfExp($str = '', $v = '', $extra = '')
		{
		
			$parsearr = array('var' => PARSING_ISVAR,
					  'save' => PARSING_ISCONST, 'func' => PARSING_ISCONST);
		   $doIfExp = false;
			foreach ($parsearr as $name => $parsetype){
				$tt = $this->parseValue($extra, $name);
				if (!strlen($tt)) continue;
				if ($parsetype != PARSING_ISVAR){ $$name = $tt; continue; }

				if (is_array($v) &&  isset($v[$tt]) && isset($name) ) { $$name = $v[$tt]; continue; }
				else if ( isset($GLOBALS[$tt]) ){
              $fun = $GLOBALS[$tt];
              $doIfExp = $fun($v);
				  // @eval( $GLOBALS[$tt] );
				}
				else $$name = '';
			}
			
         if ($doIfExp){
			   return $this->parseStr($this->parseTag($str,$v),$v);
			}else{
			   return "";
			}
		}

		function doIf($str = '', $v = '', $extra = '')
		{
			$parsearr = array('var' => PARSING_ISVAR,
					  'save' => PARSING_ISCONST, 'func' => PARSING_ISCONST);
		   $exec_block = false;
			foreach ($parsearr as $name => $parsetype){
				$tt = $this->parseValue($extra, $name);
				if (!strlen($tt)) continue;
				if ($parsetype != PARSING_ISVAR){ $$name = $tt; continue; }

				if (is_array($v) &&  isset($v[$tt]) && isset($name) ) { $$name = $v[$tt]; continue; }
				else if ( isset($GLOBALS[$tt]) ){
					$exec_block = ($GLOBALS[$tt] !== false);
				}
				else $$name = '';
			}
			
         if ($exec_block){
			   return $this->parseStr($this->parseTag($str,$v),$v);
			}else{
			   return "";
			}
		}

		function doIfNot($str = '', $v = '', $extra = '')
		{
			$parsearr = array('var' => PARSING_ISVAR,
					  'save' => PARSING_ISCONST, 'func' => PARSING_ISCONST);
		   $exec_block = false;
			foreach ($parsearr as $name => $parsetype){
				$tt = $this->parseValue($extra, $name);
				if (!strlen($tt)) continue;
				if ($parsetype != PARSING_ISVAR){ $$name = $tt; continue; }

				if (is_array($v) &&  isset($v[$tt]) && isset($name) ) { $$name = $v[$tt]; continue; }
				else if ( isset($GLOBALS[$tt]) ){
					$exec_block = ($GLOBALS[$tt] == false);
				}
				else $$name = '';
			}

            if ($exec_block){
			   return $this->parseStr($this->parseTag($str,$v),$v);
			}else{
			   return "";
			}
		}

   		function makeChain($str = '', $v = '', $extra = '')
		{

			$parsearr = array('arr' => PARSING_ISVAR,
					  'save' => PARSING_ISCONST, 
					  'func' => PARSING_ISCONST,
					  'key' => PARSING_KEY);

            		$array_name = "";
			foreach ($parsearr as $name => $parsetype){
				$tt = $this->parseValue($extra, $name);
				if (!strlen($tt)) continue;
				
				if ($parsetype != PARSING_ISVAR){ $$name = $tt; continue; }
				
				if ($name == "arr") {$array_name = $tt;}
				if (is_array($v) &&  isset($v[$tt]) && isset($name) ) { $$name = $v[$tt]; continue; }
				else if ( isset($GLOBALS[$tt]) )  $$name = &$GLOBALS[$tt];
				else $$name = '';
				
				
			}
			
			if (empty($arr) ) $arr = array();
			if ( is_object($arr) ) $arr = get_object_vars($arr);
			else if (!is_array($arr)) $arr = array($arr);

			$size = sizeof($arr);

			if (!$size) {
				if (!empty($save)) $GLOBALS[$save] = '';
				return '';
			}

			if (empty($func) || is_object($func) || is_array($func) || (!method_exists($this, $func))) {
				$func = 'parseStrTag';
			}

			$res = '';

			$key = isset($key) ? $key : "";
            $GLOBALS["rownum_".$array_name."_".$key] = 0;
            $cont = &$GLOBALS["rownum_".$array_name."_".$key];
            //r:$cont = 0;
			$slice = 0;
            $ant = $this->getKey($arr[0], $key);
			
			$cores = "_a_cor".$key;
			$$cores = $this->colors;
			foreach ($arr as $arrv){
                //r:echo "($key)\n";
                //r:print_r($arrv);
                //r:echo "\n<BR>\n";
                if (!empty($$cores)) {
    				   $GLOBALS["_color".$key] = current($$cores);
				   if (!next($$cores))
				      reset($$cores);
        		}
				if ($ant != $this->getKey($arrv, $key)) {
					break;
				}else {
			            $slice++;		
				} 
	            $temp = safe_array_merge($v, $arrv );

				$res .= ( $func ? ( in_array($func, $this->basicstand)
							? $this->$func($str, $temp)
							: ( in_array($func, $this->stand)
						   		? $this->$func($str, $temp, $extra)
						   		: $this->$func($arrv) ) )
						: $str );
				
				if ($cont++ > $this->maxreg) break;
                if ($size != sizeof($arr)) { reset($arr); }
			}
			if ($size == sizeof($arr) ) {
			   $arr = array_slice ($arr, $slice);
                //r:echo "Remove $slice reg.\n";
            }
			
			if ( !empty($save) ) $GLOBALS[$save] = $res;
               while (strpos($res, "\'")>-1)
                $res = str_replace("\'", "'", $res);
			return $res;
		}
				
		function getKey($arr, $key) {
            if (!isset($arr) || !is_array($arr)) return "";
            if (!$key) return "";
			$akey= split ("&", $key);
			$skey = "";
			foreach($akey as $k) {
                    if (!isset($arr[$k])) continue;
			       $skey .= $arr[$k];
			}
			return $skey;
		
		}

        //r: Controle de tags dinmicas, permite o uso de variaveis para definio de tags
		function PW_VarReplace($linhas){
		   if (count($this->swapvars) > 1) {
			  $a_sep   = $this->swapvars[0];
			  $a_lista = $this->swapvars[1];

			  $a_vnome = array();
			  $a_valor = array();
			  foreach($a_lista as $k => $v){
				 $vnome = $a_sep[0] . $k .$a_sep[1];
				 $a_vnome[count($a_vnome)] = $vnome;
				 $a_valor[count($a_valor)] = $v;
			  }

			  $linhas = str_replace($a_vnome,$a_valor,$linhas);
		   }
		   return $linhas;
		}

}
?>
