<?php
  /**
   *  Funes bsicas de uso geral
   *  Mdulo ADM
   */

   /**
    * Testa tipo de requisio (GET ou POST)
    *
    * @param string $method Tipo a ser testado (GET ou POST)
    * @return boolean Retorna true se a rquisio  do tipo indicado
    */
   function reqType($method="POST"){
      $method = strtoupper($method);
      return ($method == $_SERVER['REQUEST_METHOD']);
   }

   /**
    * L parametro de requisio passado por GET ou POST
    *
    * @param string $paramName Nomedo parametro (campo) passado por GET ou POST.
    * @param mix $defaultValue Valor padro caso o campo no esteja definido
    * @param string $reqType metodo pelo qual o post foi feito, valores vlidos:<br />
    *               get   - le o parametro se este foi passado por GET     <br />
    *               post  - le o parametro se este foi passado por POST    <br />
    *               all   - le o parametro passado por GET ou POST         <br />
    * @return mix Valor ou array correspondente passado por guet ou post
    */
   function getParam($paramName,$defaultValue,$reqType="all"){
      $reqType = strtolower($reqType);
      $result = "";
      if ($reqType == 'get'){
          $result = lerGet($paramName);
      }else if ($reqType == 'post'){
         $result = lerPost($paramName);
      }else if ($reqType == 'all'){
         $result = lerGet($paramName);
         if (empty($result)){
            $result = lerPost($paramName);
         }
      }
      if (empty($result)){ $result = $defaultValue; }

      return $result;
   }

   /**
    * L parametro de requisio passado por GET ou POST
    *
    *
    * @param string $paramName Nomedo parametro (campo) passado por GET ou POST.
    * @param mix $defaultValue Valor padro caso o campo no esteja definido
    * @param string $reqType metodo pelo qual o post foi feito, valores vlidos:<br />
    *               get   - le o parametro se este foi passado por GET     <br />
    *               post  - le o parametro se este foi passado por POST    <br />
    *               all   - le o parametro passado por GET ou POST         <br />
    * @param string $opt lista de opes para tratamento da informao composta por <br />
    *               uma ou mais flags conforme a tabela abaixo: <br />
    *  <table>
    *    <tr>
    *      <td><b>t-</b></td><td>Aplicar TRIM()</td>
    *    </tr>
    *    <tr>
    *      <td><b>l-</b></td><td>Converter para minusculo</td>
    *    </tr>
    *    <tr>
    *      <td><b>u-</b></td><td>Converter para maisculo</td>
    *    </tr>
    *    <tr>
    *      <td><b>s-</b></td><td>texto seguro (escapa/substitui caracteres especiais)</td>
    *    </tr>
    *    <tr>
    *      <td><b>az-</b></td><td>converte caracteres acentuados (A-Z,a-z) para caracter comum</td>
    *    </tr>
    *    <tr>
    *      <td><b>i-</b></td><td>converter para inteiro</td>
    *    </tr>
    *    <tr>
    *      <td><b>cR-</b></td><td> Recebe Currency BRF (1.232,43) e converte para Float Mysql (1232.43)</td>
    *    </tr>
    *  </table>
    * @param int $len Tamanho mximo da string retornada
    *
    * @return string Retorna valor postado tratado na forma solicitada.
    *
    * <b>exemplo Trim+UpperCase:</b><br />
    * <code>$nome = getParam_s("nome","","post","t-u-");</code>
    *
    */
   function getParam_s($paramName,$defaultValue,$reqType="all",$opt="s-t-",$len=0){
      $result = getParam($paramName,$defaultValue,$reqType);
      if (strpos($opt,"t-") !== false){
         $result = trim($result);
      }
      if (strpos($opt,"u-") !== false){
         $result = strtoupper($result);
      }else if (strpos($opt,"l") !== false){
         $result = strtolower($result);
      }
      if (strpos($opt,"s0-") !== false){
         $result = safeText2($result);
      }
      if (strpos($opt,"s-") !== false){
         $result = safeText($result);
      }
      if (strpos($opt,"az-") !== false){
         $result = trataTxt($result);
      }

      if ($len > 0){
         $result = substr($result,0,$len);
      }

      if (strpos($opt,'cR-') !== false){
          CurrencyBRF_param($result,"");
      }

      if (strpos($opt,"i-") !== false){
         $result = (int) $result;
      }

      return $result;
   }

   /**
    * L parmetro serializado (array) passado por GET ou POST
    *
    * @param string $paramName Nomedo parametro (campo) passado por GET ou POST.
    * @param array $defaultValue Valor padro caso o campo no esteja definido
    * @param string $reqType metodo pelo qual o post foi feito, valores vlidos:<br />
    *               get   - le o parametro se este foi passado por GET     <br />
    *               post  - le o parametro se este foi passado por POST    <br />
    *               all   - le o parametro passado por GET ou POST         <br />
    * @param string $opt lista de opes para tratamento da informao composta por <br />
    *  <table>
    *    <tr>
    *      <td><b>t-</b></td><td>Aplicar TRIM()</td>
    *    </tr>
    *    <tr>
    *      <td><b>l-</b></td><td>Converter para minusculo</td>
    *    </tr>
    *    <tr>
    *      <td><b>u-</b></td><td>Converter para maisculo</td>
    *    </tr>
    *    <tr>
    *      <td><b>s-</b></td><td>texto seguro (escapa/substitui caracteres especiais)</td>
    *    </tr>
    *    <tr>
    *      <td><b>az-</b></td><td>converte caracteres acentuados (A-Z,a-z) para caracter comum</td>
    *    </tr>
    *    <tr>
    *      <td><b>i-</b></td><td>converter para inteiro</td>
    *    </tr>
    *    <tr>
    *      <td><b>cR-</b></td><td> Recebe Currency BRF (1.232,43) e converte para Float Mysql (1232.43)</td>
    *    </tr>
    *  </table>
    * @param int $len Tamanho mximo da string retornada
    *
    * @return array Retorna o array correspondente ou o
    * <b>exemplo Trim+UpperCase:</b><br />
    * <code>$nomes = getArrayParam("nome","","post","t-u-");</code>
    */
   function getArrayParam($paramName,$defaultValue=false,$reqType="all",$opt="s-t-",$len=0){
      if ($defaultValue===false){ $defaultValue= array(); }
      $result = getParam($paramName,false,$reqType);

      if ($result === false || !is_array($result)){
         $result = $defaultValue;
      }else{
         if (strpos($opt,"t-") !== false){
            array_walk($result,'trim_param');
         }
         if (strpos($opt,"u-") !== false){
            array_walk($result,'uplow_param','u');
         }else if (strpos($opt,"l-") !== false){
            array_walk($result,'uplow_param','l');
         }
         if (strpos($opt,"s-") !== false){
            array_walk($result,'safe_param');
         }
         if (strpos($opt,"s0-") !== false){
            array_walk($result,'safe_param2');
         }
         if (strpos($opt,"az-") !== false){
            array_walk($result,'az_param');
         }

         if ($len > 0){
            array_walk($result,'trunc_param',$len);
         }

         if (strpos($opt,"cR-") !== false){
             array_walk($result,"CurrencyBRF_param");
         }

         if (strpos($opt,"i-") !== false){
            array_walk($result,'int_param');
         }
      }
      return $result;
   }

   /**
    * Testa se uma varivel de sesso esta definida (existe)
    *
    * @param String $varName Nome da varivel de sesso a ser testada
    *
    * @return Boolean Retorna true se a varivel informada estiver definida na sesso
    */
   function inSession($varName){
      return isset($_SESSION[$varName]);
   }

   /**
    * Seta valor para varivel de sesso
    *
    * @param String $varName  Nome da varivel de sesso
    * @param mix $data Valor ou estrutura a ser armazenado.
    */
   function putSession($varName,$data){
      $_SESSION[$varName] = $data;
   }

   /**
    * Retorna varivel de sesso
    *
    * @param String $varName Nome da varivel de sesso a ser retornada
    * @return mix Retorna o valor ou estrutura atribuido  varivel de sesso informada
    */
   function getSession($varName){
      return $_SESSION[$varName];
   }

   /**
    * Remove varivel da sesso
    *
    * @param String $varName Nome da varivel a ser liberada
    */
   function freeSession($varName){
      unset($_SESSION[$varName]);
   }

  /**
   * Processa texto recebido por GET ou POST protegendo contra injeo de SQL
   *
   * @param mix $texto contedo a ser verificado
   * @return mix Retorna contedo seguro
   */
  function safeText($texto){
     if (!get_magic_quotes_gpc()){
        $texto =  mysql_escape_string($texto);
     }else{
       $texto = str_replace("'","",$texto);
       $texto = str_replace('"',"`",$texto);
     }
     return $texto;
  }

  /**
   * Processa texto recebido por GET ou POST protegendo contra injeo de SQL
   * se escapar caracteres
   * @param mix $texto contedo a ser verificado
   * @return mix Retorna contedo seguro
   */
  function safeText2($texto){
     $texto = str_replace("'","",$texto);
     $texto = str_replace('"',"`",$texto);
     return $texto;
  }

  /** Auxiliar getParam() */
  function lerGet($paramName){
      if (isset($_GET[$paramName])){
         return $_GET[$paramName];
      }else{
         return false;
      }
  }
  /** Auxiliar getParam() */
  function lerPost($paramName){
      if (isset($_POST[$paramName])){
         return $_POST[$paramName];
      }else{
         return false;
      }
  }


   /** Auxiliar de getArrayParam() */
   function trim_param(&$item,$key){
      $item = trim($item);
   }
   /** Auxiliar de getArrayParam() */
   function uplow_param(&$item,$key,$opt){
      if ($opt=='u'){
         $item = strtoupper($item);
      }else if ($opt='l'){
         $item = strtolower($item);
      }
   }
   /** Auxiliar de getArrayParam() */
   function safe_param(&$item,$key){
      $item = safeText($item);
   }
   /** Auxiliar de getArrayParam() */
   function safe_param2(&$item,$key){
      $item = safeText2($item);
   }
   /** Auxiliar de getArrayParam() */
   function az_param(&$item,$key){
      $item = trataTxt($item);
   }
   /** Auxiliar de getArrayParam() */
   function trunc_param(&$item,$key,$len){
      $item = substr($item,0,$len);
   }
   /** Auxiliar de getArrayParam() */
   function int_param(&$item,$key){
      $item = (int) $item;
   }

   function CurrencyBRF_param(&$item,$key){
      $item = str_replace(".", "", $item);
      $item = str_replace(",", ".", $item);
   }
