<?php
  /*
   Funoes para manipulao de arquivos
   Autor.: Alexandre P. Trindade
   Inicio: 15/09/2004
   Copyright Uniware Informtica
  */

   /*  Alexandre P. Trindade
    *  Data: 19/03/2009
    *  getDirList  - Retorna a lista de arquivos de uma pasta
    *
    *  @parametros
    *    path       - Pasta alvo da pesquisa
    *    mask       - Mascara que define o grupo de arquivos a serem pesquisados
    *    dateMask   - Mascara de formatao de datas que ser usada para fitros (minDate e maxDate)
    *    minDate    - Data inicial de um filtro, se maxDate no for definido (false) sero incluidos
    *                 todos os arquivos com data de ultima modificao maior ou igual minDate.
    *    maxDate    - Data final de um filtro, se minDate no for especificado (false) sero listados
    *                 todos os arquivos com data menor ou igual a maxDate.
    *    minSize    - Tamanho minimo em bytes para um filtro, se maxSize no for especificado sero
    *                 listados todos os arquivos com tamanho maior ou igual a minSize
    *    maxSize    - Tamanho maximo de arquivo para filtro, se minSize nao for definido sero
    *                 listados todos os arquivos com tamenho menor ou igual a maxSize.
    *
    *   @exemplo de chamada:
    *       O exemplo abaxo retorna todos os pdfs da pasta resultados com data de ultima modificao
    *       anterior a 01/02/2009
    *       $a_files = getDirList("/resultados","*.pdf","Ymd",false,"20090201");
    */
   function getDirList($path,$mask="*.*",$dateMask="YmdHis",$minDate=false, $maxDate=false,$minSize=false,$maxSize=False){
      $result = array();
      if (empty($mask)){ $mask = "*.*"; }
      if (empty($path)){ $path = "."; }
      if (empty($dateMask)){ $dateMask = "YmdHis"; }

      if ($handle = opendir($path)) {
          while (false !== ($filename = readdir($handle))) {
            if ($filename != "." && $filename != "..") {
               if (function_exists("fnmatch")){
                  $math_func = "fnmatch";
               }else{
                  $math_func = "win_fnmatch";
               }
               if ($math_func($filename,$mask)){
                  $pi = pathinfo( "/$filename" );
                  if (is_dir("$path/$filename")) {
                     $fileInfo = Array("name" => $pi["basename"], "type" => "D", "size" => "", "time" => "");
                     array_push($result,$fileInfo);
                  }else{
                     $inclui = true;
                     $fs = filesize("$path/$pi[basename]");
                     $ft = filemtime("$path/$pi[basename]");
                     // Checa os filtros se houver
                     if ($minDate !== false){
                        $inclui = (date($dateMask,$ft) >=  $minDate);
                     }

                     if ($inclui && ($maxDate !== false)){
                        $inclui = (date($dateMask,$ft) <=  $maxDate);
                     }

                     if ($inclui && ($minSize !== false)){
                        $inclui = ($minSize >=  $minSize);
                     }

                     if ($inclui && ($maxSize !== false)){
                        $inclui = ($maxSize <=  $maxSize);
                     }

                     if ($inclui){
                        $fileInfo = array("name" => $pi["basename"],
                                          "type" => "A",
                                          "size" => $fs,
                                          "time" => $ft);
                        array_push($result,$fileInfo);
                      }
                  }
               }
            }
          }
          closedir($handle);
      }

      return $result;
   }

   function str_match($exp,$val){
      if (preg_match($exp, $val)) {
         return true;
      }else{
         return false;
      }
   }

   function win_fnmatch($fileName,$mask){
      $result = false;
      $inf = pathinfo($fileName);
      $mp  = split('\.',$mask);

      if (is_array($mp)){
         if (count($mp) == 1){
            $result = ($mp[0] == '*' || strtolower($mp[0]) == strtolower($inf["extension"]));
         }else{
            $e = ($mp[0] == '*' || strtolower($mp[0]) == strtolower($inf["extension"]));
            $n = ($mp[1] == '*' || strtolower($mp[1] == strtolower($inf["filename"])));
            $result = ($e && $n);
         }
      }else{
         return true;
      }

      return $result;
   }


   /*  Alexandre P. Trindade
    *  Data: 03/10/2011
    *  listFolders  - Retorna a lista de subpastas de uma pasta qualquer
    *
    *  @parametros
    *    path       - Pasta alvo da pesquisa
   */
   function listFolders($path){
      $result = array();
      if (empty($path)){ $path = "."; }

      if ($handle = opendir($path)) {
          while (false !== ($filename = readdir($handle))) {
            if ($filename != "." && $filename != "..") {
               $pi = pathinfo( "/$filename" );
               if (is_dir("$path/$filename")) {
                  $fileInfo = Array("name" => $pi["basename"], "type" => "D", "size" => "", "time" => "");
                  array_push($result,$fileInfo);
               }
            }
          }
          closedir($handle);
      }
      return $result;
   }

   /*
      Lista apenas os arquivos de uma pasta
   */
   function listFiles($path,$mask="*.*"){
      $result = array();
      if (empty($mask)){ $mask = "*.*"; }
      if (empty($path)){ $path = "."; }

      if ($handle = opendir($path)) {
          while (false !== ($filename = readdir($handle))) {
            if ($filename != "." && $filename != "..") {
               if (uZip::testaNome($filename,$mask)){
                  $pi = pathinfo( "/$filename" );
                  if (is_dir("$path/$filename")) {
                     // nao faz nada (descarta)
                  }else{
                     $inclui = true;
                     $fs = filesize("$path/$pi[basename]");
                     $ft = filemtime("$path/$pi[basename]");

                     $fileInfo = array("name" => $pi["basename"],
                                       "type" => "A",
                                       "size" => $fs,
                                       "time" => $ft);
                     array_push($result,$fileInfo);
                  }
               }
            }
          }
          closedir($handle);
      }

      return $result;
   }


  //r: L uma arquivo texto qualquer e retorna o contedo do arquivo
  //r: Retorna Falso se a funo falhar.
  function GetTextFromFile($filename){
    $texto = false;
	if (file_exists($filename)) {
	   $hand_o = @fopen($filename,"rb");
	   if (!($hand_o === false)) {
          $texto = fread( $hand_o, filesize($filename) );
		  fclose($hand_o);
	   }
	}
    return $texto;
  }

  //r: Gera um identificador de arquivo exclusivo
  function NewFileId() {
    return "_" .date("YmdHis") . mt_rand( 0, 32848 );
  }

  function RootDir(){ return $_SERVER["DOCUMENT_ROOT"]; }

  //r: Salva o contedo de mbuff em um arquivo
  function SaveToFile($buff,$filename){
     $hand = @fopen($filename,"wb");
	 if ($hand !== false){
	    fwrite($hand,$buff);
		fclose($hand);
		return true;
	 }else{
	    return false;
	 }
  }

  /* Processa um arquivos de template, subistituindo suas variaveis pelos
     valores correspondentes.
	 Parametros:
	    $var_lst   - Array contendo a lista de valores para cada varivel.
		             O ndice do array deve corresponder ao nome da variavel no arquivo.
		$var_delim - delimitadores de identificao de variveis.
		             Arrayu de duas posies contendo prefixo na posio zero e sufixo na posiao 1
  */
  function GetDocFromTemplate($var_lst,$var_delim,$filename){
      global $getTemplateDebug;
      $texto = GetTextFromFile($filename);
	   if ($texto !== false){
	       foreach($var_lst as $k => $v){
		      $vn = $var_delim[0] . $k . $var_delim[1];
			   $texto = str_replace($vn,$v,$texto);
		   }
	   }
	   return $texto;
  }

  function ExtractFilePath($s){
	  $r = pathinfo($s);
	  return $r["dirname"];
  }

  function ExtractFileName($s){
	  $r = pathinfo($s);
	  return $r["basename"];
  }

  function ExtractFileExt($s){
	  $r = pathinfo($s);
	  return $r["extension"];
  }

  define("NO_SUBDIR",false);
  define("SINGLE_NAMES",true);

  //r: Retorna a lista de arquivos e sub pastas de um diretrio
  //r: Obsoleto usar getDirList()
  function DirList($dir,$mask,$sb_dir = true,$single_names = false){
	 $KBYTE = 1024;
     $MEGABYTE = $KBYTE * 1024;
     $GIGABYTE = $MEGABYTE * 1024;

     $result = Array();
     if (empty($mask)) {
		 $mask = "*.*";
	 }

	//r: busca por aquivos e pastas que combinem com o padro ($mask)
	$i = 1;
	$result[0] = Array(0,"");
	$total_bytes = 0;
	foreach (glob("$dir/$mask") as $filename){
	  if ($filename !== "." && $filename !== ".."){
		  if (is_dir("$dir/$filename")) {
			 if ($sb_dir === true){
				if ($single_names){
			       $result[$i] = Array("name" => ExtractFileName($filename), "type" => "D");
				}else{
			       $result[$i] = Array("name" => $filename, "type" => "D");
				}
			    $i += 1;
			 }
		  }else{
			 $fs = filesize($filename);
			 if ($single_names){
				 $result[$i] = Array("name" => ExtractFileName($filename), "type" => "A",
									 "size" => $fs,
									 "time" => filemtime($filename) );
			 }else{
				 $result[$i] = Array("name" => $filename, "type" => "A",
									 "size" => $fs,
									 "time" => filemtime($filename) );
			 }
			 $i += 1;
			 $total_bytes += $fs;
		  }
	  }
	}
	$allSize = "";
	$r = $total_bytes;
	$lab = " Bytes";
	if ($r >= 1024){
	   $r = ($r / 1024);
	   $lab = " KB  ($total_bytes Bytes)";
	}

	if ($r >= 1024){
	   $r = ($r / 1024);
	   $lab = " MB  ($total_bytes Bytes)";
	}

	if ($r >= 1024){
	   $r = ($r / 1024);
	   $lab = " GB  ($total_bytes Bytes)";
	}
    $allSize = number_format($r,2,",","") . $lab;

	$result[0] = Array("COUNT" => $i-1,"DIR" => $dir,"SIZE" => $allSize,"BYTES" => $total_bytes);
	return $result;
  }

  /* retorna a lista simples de arquivos de um diretrio (no inclui subdiretorior).
     Os nomes retornados nao possuem o path
     Obsoleto - Usar getDirList()
  */
  function GetFileList($dir,$mask,$insert_path = ""){
	 $result = array();
    $aux = DirList($dir,$mask, NO_SUBDIR, SINGLE_NAMES);
    $r = array_shift($aux);
	reset($aux);
    foreach($aux as $k => $v){
      if (empty($insert_path)){
		   $result[] = $v["name"];
		}else{
		   $result[] = $insert_path . "/" . $v["name"];
		}
	 }
	 return $result;
  }

  //r: Apaga arquivo ou lista de arquivos - aceita mascara  (*.*, *.qqc, qqc.*) ou nome de diretrio
  function Erase($param){
     if (is_string($param)) {
        if (is_file($param)) {
  		   if (@unlink($param)){
               return true;
   		   }else{
   		      return false;
   		   }
        } else if (is_dir($param)) {
           $ok = Erase("$param/*.*");
           if (! $ok) {
               return false;
           }
           //r: return rmdir($param);  //r: remove o diretrio
        } else {
           $matching = glob($param);
           if ($matching === false) {
              //r: trigger_error(sprintf('Nenhum arquivo %s encontrado !', $param), E_USER_WARNING);
              return true; //r: false;  //r: Retorna trus se o arquivo no existir
           }
           $rcs = array_map('Erase', $matching);
           if (in_array(false, $rcs)) {
               return false;
           }
        }
    } else if (is_array($param)) {
        $rcs = array_map('Erase', $param);
        if (in_array(false, $rcs)) {
           return false;
        }
    } else {
        trigger_error('Parametro #1 deve ser um nome de arquivo, mascara, nome de diretriorn ou array de nomes ou mascaras', E_USER_ERROR);
        return false;
    }

    return true;
 }

 /*
    faz download de arquivos sem expor a url de origem
  */
 function download_hidden($filename,$dwName="") {
     // Com SaveDlg
     $size = filesize($filename);
     $file_info  = pathinfo( $filename );
     $dwName = (empty($dwName))?  $file_info["basename"]:$dwName;

	  header("Content-type: application/" . $file_info["extension"]);
	  header("Content-Disposition: attachment; filename=$dwName");
     header("Content-Length: $size");
	  readfile($filename);
 }

 function send_doc_hidden($filename) {
     // Abrir no browser
     $size = filesize($filename);
     $file_info  = pathinfo( $filename );
	  header("Content-type: application/" . $file_info["extension"]);
     header("Content-Length: $size");
	  readfile($filename);
 }

 function send_linked_hidden($filename, $ext) {
     // Abrir no browser
 	  header("Content-type: application/$ext");
     // header("Content-Length: $size");
	  readfile($filename);
 }

 function download_linked_hidden($filename,$dwName,$ext) {
     // Com SaveDlg
 	  header("Content-type: application/$ext");
	  header("Content-Disposition: attachment; filename=$dwName");
     // header("Content-Length: $size");
	  readfile($filename);
 }

 /**
 * Retorna informaes do Browsers
 */
class browser{
   /**
    * Recorta string
    * @param string $s String a ser recortada passada por referncia
    * @param string $sep caracter separador, que indica o ponto de corte
    * @return string Retorna a poro removida da string informada
    */
   static function CutStr(&$s,$sep="|"){
      $p = strpos($s,$sep);
      if ($p===false){
         $r = $s; $s = "";
         return $r;
      }else{
         $r = substr($s,0,$p);
         $s = substr($s,$p+1);
         return $r;
      }
   }

   /**
    * Testa se a sequencia $n existe existe dentro da string $s
    * @param string $n Sequencia procurada
    * @param string $s Cadeia pesquisada
    * @return int posio da sequencia dentro da cadeia.
    */
   static function hitName($n,$s){
      $r = strpos(strtolower($s),strtolower($n));
      if ($r===false){
         return -1;
      }else{
         return $r;
      }
   }

   /**
    * Extrai Nome e verso do navegador da string HTTP_USER_AGENT
    */
   static function getBwNameVer($s){
      $result = false;
      if (browser::hitName("chrome",$s)>=0){
         $p = browser::hitName("chrome",$s);
         $r = substr($s,$p);
         $c = browser::CutStr($r," ");
         $result["name"] = browser::CutStr($c,"/");
         $result["ver"]  = browser::CutStr($c," ");
      }else if (browser::hitName("msie",$s)>=0){
         $p = browser::hitName("msie",$s);
         $r = substr($s,$p);
         $c = browser::CutStr($r,";");
         $result["name"] = browser::CutStr($c," ");
         $result["ver"]  = browser::CutStr($c,";");
      }if (browser::hitName("firefox",$s)>=0){
         $p = browser::hitName("firefox",$s);
         $r = substr($s,$p);
         $c = browser::CutStr($r," ");
         $result["name"] = browser::CutStr($c,"/");
         $result["ver"]  = $c;
      }else if (browser::hitName("netscape",$s)>=0){
         $p = browser::hitName("netscape",$s);
         $r = substr($s,$p);
         $c = browser::CutStr($r," ");
         $result["name"] = browser::CutStr($c,"/");
         $result["ver"]  = $c;
      }
      return $result;
   }

   /**
    * Retorna um array contendo as informaes do browser com a seguinte estrutura <br>
    * $["padrao"]           - Padro de browser (geramlemte Mozilla) <br>
    * $["geracao"]          - Gerao de browser  <br>
    * $["know"]             - (boolean) Recongecido pela <br>
    * $["browser"]["name"]  - Nome do Broser (Chrome/FireFox/MSIE) <br>
    * $["browser"]["ver"]   - Verso do browser <br>
    */
   static function agentInfo(){
      $result = array();
      $b = $_SERVER["HTTP_USER_AGENT"];
      $g = browser::CutStr($b," ");
      $mozilla = browser::CutStr($g,'/');
      $result["padrao"]  = $mozilla;
      $result["geracao"] = $g;

      $c = strtolower($result["padrao"]);
      if ("mozilla"==$c){
         // Tipos conhecidos
         $result["know"] = true;
         $result["browser"] = browser::getBwNameVer($b);
      }else{
         // Tipo desconhecido
         $result["know"] = false;
         $result["browser"] = array();
      }
      return $result;
   }

}

global $BROWSER;
$BROWSER = browser::agentInfo();

?>
