<?php
  /* ---------------------------------------------------------------------------
      uLib Versao 1.0
	  Autor: Alexandre P. Trindade
	  Copyright 2004 - Uniware Acessoria e Consultoria de Sistemas LTDA.
      Inicio: 15/09/2004
	 ---------------------------------------------------------------------------


	 15/06/2011 - Alexandre P. Trindade
	              1 - Implementado o evento OnInvalidValue chamado sempre
                      que um valor invlido for passado para o campo.

                      interface da funo para manipular o evento:

                      function <nomeDaFuncao>($fieldName,$fieldType,&$value)
                      Recebe:
                        $fieldName    - Nome do campo
                        $fieldType    - Tipo do campo
                        &$value       - valor atribuido passado por referncia
                                        o manipulador pode alterar este valor
                                        mas ter que garantir que o valor
                                        retornado esteja correto e seja vlido


	              2 - Implementada critica de valores para os segintes campos:

	              int,real         Converte valor para tipo correspondente
                                   e dispara evento OnInvlaidValue se estiver definido

                  date_br,
                  datetime_br,
                  timestamp_br     Valida as datas e coloca no formato aceito pelo banco de dados
                                   se uma data invlida for passada ser setada para NULL a menos
                                   que o evento OnInvalidValue esteja definido, se estiver passa
                                   a deciso para o procedimento

	 ---------------------------------------------------------------------------

  */

  /*{$NC+}*/
  uses("objects","lib_seg");
  /*{$NC-}*/
  /*{$CB+}
  uses("objects");
  {$CB-}*/


  /*
    FieldType
    ---------

    exp		      Expresso - qualquer expresso SQL vlida

    text,
    string		   Campos TEXT,CHAR,VARCHAR

    string_safe    Campos TEXT,CHAR,VARCHAR com aspas escapadas

    long,
    int,
    integer		   Campos INT

    currency,
    double,
    real		     Campos DECIMAL/DOUBLE

    real_br          Campos DECIMAL/DOUBLE no padro brasileiro de formatao

    date		     Campos DATE/TIME/TIMESTAMP (sem checagem)

    date_br          Campo DATE valor passado no formato brasileiro
    datetime_br      Campo DATETIME valor passado no formato brasileiro
    timestamp_br     Campo TIMESTAMP valor passado no formato brasileiro
  */

  /**
  *  Executa instrues SQL de "INSERT|UPDATE|DELETE"
  */
  class uMySQLSql extends uObject {
     /**
      * conexao Handle da conexo MySQL a ser utilizada
      * @var int
      */
     var $conexao;
     /**
      * Id do ultimo registro inserido
      * @var int
      */
     var $lastID;
     /**
      * Resgata mensagens informando o ultimo erro
      * @var string
      */
     var $lastError;
     /**
      *  Nome da tabela para insert/update/delete
      * @var string
      */
     var $table;
     /**
      * Operao "INSERT"|"UPDATE"|"DELETE"
      * @var string
      */
     var $command;
     /**
      * Condio de filtro (Restries da clausula Where)
      * @var string
      */
     var $where;
     /**
      * Private array contendo a lista com as definies dos campos
      * @var array
      */
     var $fFieldDef;
     /**
      *  Texto contendo a ultima SQL executada
      * @var string
      */
     var $sqlCheck;
     /** Private */
     var $preSQL;               //r: SQL de consulta para retorno de linhas afetadas
     /**
      *  Lista dos campos relacionados que no entram no update (chaves primrias)
      * @var array
      */
     var $NoUpdateFields;
     /**
      * Evento OnEmptyDate(obj,FieldName,FieldValue)  disparado sempre que uma data  gravada vazia,zerada ou NUL
      * @var function
      */
     var $OnEmptyDate;
     /**
      * Obsoleto (default false) - define se um comando commit deve ser enviado ao banco apos um INSERT/UPDATE/DELETE (banco InnoDB)
      * @var boolean
      * @deprecated
      */
     var $InnoDB;
     /**
      * Array contendo mascaras de entrada para convero de datas (date,datetime,timestamp)
      * @var string
      */
     var $dateMasks;

     /**
      * evento chamado quando um valor invalido  atribuido a um campo
      * sintaxe: function($fieldName,$fieldType,&$value)
      * @var function
      */
     var $OnInvlaidValue;


    function uMySQLSql() {
       $cn = "uMySQLSql";
       $name      = (func_num_args() > 0)?  func_get_arg(0):"nonamed-$cn";
       $classPath = (func_num_args() > 1)?  "$cn.".func_get_arg(1):$cn;

       $this->uObject($name,$classPath); //r: inherited

       $this->conexao   = -1;
       $this->command   = "";
       $this->lastError = "";
       $this->where     = "";
       $this->table     = "";
       $this->fFieldDef = array();
       $this->sqlCheck  = "";
       $this->lastID    = 0;
       $this->preSQL    = "";
       $this->NoUpdateFields = array();
       $this->InnoDB      = false;

       $this->dateMasks = array("date" => "d/m/a","datatime"=>"d/m/a h:i:s","timestamp"=>"d/m/a h:i:s");
       $this->OnInvlaidValue = "";
    }

    /** Reseta definies no objeto para reutilizao. */
    function Renew(){
       $this->command   = "";
       $this->lastError = "";
       $this->where     = "";
       $this->table     = "";
       $this->fFieldDef = array();
       $this->sqlCheck  = "";
       $this->lastID    = 0;
       $this->preSQL    = "";
       $this->NoUpdateFields = array();
    }

    /** Limpa o buffer de definio de campos */
    function Clear(){
	     unset($this->fFieldDef);
	     $this->fFieldDef = array();
	}

    /** Envia um commit ao banco se estiver acessando um banco InnoDB -
     * fora de uso (transao implementada na classe de conexo)
     */
    function Commit(){
       if ($this->InnoDB){ mysql_query("COMMIT", $this->conexao); }
    }

    /** Executa um SQL genrico ou script */
    function ExecSQL($SQL){
	   $r = mysql_query($SQL, $this->conexao);
       if (!$r){
          $this->lastError = mysql_error();
       }else if ($this->InnoDB){
          $this->Commit();
       }
       return !($r===false);
    }

    function _DisabledField($fieldName){
	   if (count($this->NoUpdateFields) < 1){
	      return false;
	   }else{
	     reset($this->NoUpdateFields);
	     $result = false;
	     foreach($this->NoUpdateFields as $k => $v){
		    if ($fieldName == $v) {
			   $result = true;
			}
		 }
		 return $result;
	   }
	}

   function _ValidFieldType($tipo){
	   $result = false;
	   switch ($tipo) {
		   case "exp" :
		   case "text" :
		   case "string" :
		   case "string_safe" :
		   case "long" :
		   case "int" :
		   case "integer" :
		   case "real" :
		   case "real_br" :
		   case "double" :
		   case "currency" :
		   case "date" :
		   case "date_br" :
		   case "datetime_br" :
		   case "timestamp_br" :
			    $result = true;
		        break;
	   }
       return $result;
	}

	function _GetSQLValStr($fieldName,$theValue, $theType, $theDefinedValue = "", $theNotDefinedValue = "")
	{
		$theValue = (!get_magic_quotes_gpc()) ? addslashes($theValue) : $theValue;
		$m = "";
		switch ($theType) {
		case "exp" :
		  $theValue = ($theValue != "") ? $theValue : "NULL";
		  break;
		case "string_safe" :
		  $theValue = slqProtect($theValue);
		case "text":
		case "string":
		case "date":
		  $theValue = ($theValue != "") ? "'" . $theValue . "'" : "NULL";
		  break;
        case "date_br":
          $m  = $this->dateMasks["date"];  // formato de entrada
          $mf = "a-m-d";                   // formato de gravao
          if (!dataValida($theValue,$m)){
             if (!empty($this->OnInvlaidValue)){
                $val = $theValue;
                $cmd = $this->OnInvlaidValue;
                $cmd($fieldName,$theType,$val);
                if ($val == $theValue){
                   $theValue = "NULL";
                }else{
                  $theValue = $val;
                }
             }else{
                $theValue = "NULL";
             }
          }

          if ($theValue != "NULL"){
              $theValue = convertData($theValue,$m,$mf);
              $theValue = "'$theValue'";
          }
          break;
        case "datetime_br" :
          $m  = $this->dateMasks["datetime"];    //r: formato de entrada
          $mf = "a-m-d h:i:s";                   //r: formato de gravao
        case "timestamp_br" :
          if (empty($m)){
             $m  = $this->dateMasks["timestamp"]; //r: formato de entrada
             // $mf = "amdhis";
             $mf = "a-m-d h:i:s";
          }

          if (!dataValida($theValue,$m)){
             if (!empty($this->OnInvlaidValue)){
                $val = $theValue;
                $cmd = $this->OnInvlaidValue;
                $cmd($fieldName,$theType,$val);
                if ($val == $theValue){
                   $theValue = "NULL";
                }else{
                  $theValue = $val;
                }
             }else{
                $theValue = "NULL";
             }
          }

          if ($theValue != "NULL"){
              $theValue = convertData($theValue,$m,$mf);
              $theValue = "'$theValue'";
          }
          break;
		case "long":
		case "int":
		case "integer":
		  if (!empty($theValue)){
             $old = $theValue;
             $theValue = (int) $theValue;

             if (empty($theValue) && !empty($this->OnInvlaidValue)){
                $val = $old;
                $cmd = $this->OnInvlaidValue;
                $cmd($fieldName,$theType,$val);
                if ($val == $old){
                   $theValue = "NULL";
                }else{
                   $theValue = $val;
                }
             }
          }
		  $theValue = ($theValue != "") ? intval($theValue) : 0;
		  break;
		case "currency":
		case "double":
		case "real":
		  if (!empty($theValue)){
              $old = $theValue;
              $theValue = (float) $theValue;

             if (empty($theValue) && !empty($this->OnInvlaidValue)){
                $val = $old;
                $cmd = $this->OnInvlaidValue;
                $cmd($fieldName,$theType,$val);
                if ($val == $old){
                   $theValue = 0;
                }else{
                   $theValue = $val;
                }
             }
          }

		  $theValue = ($theValue != "") ? doubleval($theValue) : 0;
		  break;
		case "real_br":
		  if (!empty($theValue)){
              $old = $theValue;
              $theValue = str_replace(".","",$theValue);
              $theValue = str_replace(",",".",$theValue);
              if (empty($theValue)){ $theValue = 0; }
              $theValue = (float) $theValue;

             if (empty($theValue) && !empty($this->OnInvlaidValue)){
                $val = $old;
                $cmd = $this->OnInvlaidValue;
                $cmd($fieldName,$theType,$val);
                if ($val == $old){
                   $theValue = "NULL";
                }else{
                   $theValue = $val;
                }
             }
          }

		  $theValue = ($theValue !== "") ? doubleval($theValue) : "NULL";
		  break;
		case "defined":
		  $theValue = ($theValue != "") ? $theDefinedValue : $theNotDefinedValue;
		  break;
		}
	    return $theValue;
	}

    function _GerRowsAfec(){
	   $result = 0;
	   $SQL = "SELECT COUNT(0) AS nrows FROM $this->table";
	   if (!empty($this->where)){
	      $SQL .= " WHERE $this->where";
	   }
       $this->preSQL = $SQL;
       $rs = mysql_query($SQL,$this->conexao);
	   if (($rs !== false) && ($rec = mysql_fetch_assoc($rs))) {
	      $result = $rec["nrows"];
	   }
	   mysql_free_result($rs);
	   return $result;
	}

    //r: Reduz tamanho das tabelas liberando espao dos registros excluidos
    function _PackTable() {
	    $tmp_tab  = "tmp_" . $this->table;
		$orig_tab = $this->table;
		$old_tab  = $this->table . "_old";
        $result = 0;   //r: Nenhum erro processo ok

		$SQL_1 = "CREATE TABLE $tmp_tab SELECT * FROM $orig_tab";
		$SQL_2 = "RENAME TABLE $orig_tab TO $old_tab, $tmp_tab TO $orig_tab";
		$SQL_3 = "DROP TABLE $old_tab";

		//r: 1 - Cria nova tabela
		$r = mysql_query($SQL_1, $this->conexao);
		if (!$r){
		   $result = 1;   //r: Create Error
		}else{
		   //r: Renomeia tabelas
           $r = mysql_query($SQL_2, $this->conexao);
		   if (!$r){
		      $result = 2;   //r: Rename Error
		   }else{
		      $r = mysql_query($SQL_3, $this->conexao);
			  if (!$r){
		         $result = 3;   //r: DROP Error
			  }
		   }
		}
        if ($result > 0){ $this->lastError = mysql_error(); }
		return $result;
	}

   /**
    *     Adiciona informaes. Use AddFieldVal para especificar os campos ($field_name)
    * a serem inseridos ou alterados, e seus respectivos tipos ($filed_type) e valores ($value)
    *
    * @param String $field_name  Nome do campo
    * @param String $field_type  Tipo do campo que pode ser:
    *                            "exp"          - Expresso, qualquer expresso SQL vlida.
    *
    *                            "text",
    *                            "string"       - Campos TEXT,VARCHAR ou CHAR.
    *
    *                            "string_safe"  - Campos TEXT,CHAR,VARCHAR com aspas escapadas.
    *
    *                            "long","int",
    *                            "integer"      - Campo numrico do tipo Inteiro.
    *
    *                            "currency",
    *                            "double",
    *                            "real"         - Campo numrico do tipo Decimal.
    *
    *                            "real_br"      - Campo decimal informado no padro brasileiro de formatao.
    *
    *                            "date"         - Campo do tip Date, Time ou Timestamp sem formatao.
    *
    *                            "date_br"      - Campo DATE valor passado no formato brasileiro (c/ valido).
    *                            "datetime_br"  - Campo DATETIME valor passado no formato brasileiro (c/ validao).
    *                            "timestamp_br" - Campo TIMESTAMP valor passado no formato brasileiro (c/ validao).
    * @param Mix $value valor que ser atribuido ao campo.
    * @return Boolean Retrona true se a operao for bem sucedida ou false em caso de falha.
    *                 No caso de falha a descrio do erro podera ser recuperada por lastError
    */
   function AddFieldVal($field_name,$field_type,$value){
       $field_type = strtolower($field_type);
	   if ($this->_ValidFieldType($field_type)){
          $this->fFieldDef[$field_name] = array("valor" => $value, "tipo" => $field_type);
	      return true;
	   }else{
		  $this->lastError = "Tipo de campo invlido '$field_type'";
	      return false;
	   }
	}

    function SetFieldValue($field_name,$value){
	  if (array_key_exists($field_name,$this->fFieldDef)){
	     $tmp = $this->fFieldDef[$field_name];
		 $tmp["valor"] = $value;
		 $this->fFieldDef[$field_name] = $tmp;
	     return true;
	  }else{
	    return false;
	  }
	}

    /**
     * Executa a instruo padro indicada em command.
     * @return boolean True se a operao for bem sucedida ou false em caso de falha.
     *                 Em caso de falha o rerro pode ser recuperado por lastError.
     */
    function Execute(){
	   if (empty($this->table)){
	      $this->lastError = "Nome da tabela no informado !";
		  return false;
	   }else{
		   $this->command = strtoupper($this->command);
		   $result = true;
		   switch ($this->command){
			 case "INSERT"  :
                  $flds = ""; $fval = "";
			      foreach($this->fFieldDef as $fnome => $finfo){
                     $flds = (empty($flds))?  $fnome: "$flds,$fnome";
				     $fv   = $this->_GetSQLValStr($fnome,$finfo["valor"],$finfo["tipo"]);
                     $fval = (empty($fval))?  $fv:"$fval,$fv";
			      }
				  $SQL  = "INSERT INTO $this->table ($flds) VALUES ($fval)";
				  $this->sqlCheck = $SQL;
			      $r = mysql_query($SQL, $this->conexao);
				  if ($r === false){
					 $this->lastError = mysql_error();
				     $result = false;
				  }else{
                     $this->Commit();
					 $this->lastID = mysql_insert_id($this->conexao);
					 $result = true;
				  }
				  break;
			case "UPDATE"  :
				 $ra = $this->_GerRowsAfec();
				 $SQL  = "UPDATE $this->table SET ";
			     $flds = "";
			     foreach($this->fFieldDef as $fnome => $finfo){
				    if (!$this->_DisabledField($fnome)) {
					    $fv  = $this->_GetSQLValStr($fnome,$finfo["valor"],$finfo["tipo"]);
				        $flds = empty($flds)? "":"$flds,";
					    $flds .= "$fnome = $fv";
					}
				  }
				  $SQL .= $flds . " ";
				  if (!empty($this->where)){
				      $SQL .= "WHERE $this->where";
				  }
				  $this->sqlCheck = $SQL;
			      $r = mysql_query($SQL, $this->conexao);
				  if ($r === false){
				 	  $this->lastError = mysql_error();
				      $result = false;
				  }else{
                      $this->Commit();
					  $result = $ra;
				  }
				  break;
			case "DELETE"  :
				  $ra = $this->_GerRowsAfec();
				  $SQL  = "DELETE FROM $this->table";
			      if (!empty($this->where)){
					 $SQL  .= " WHERE $this->where";
				  }
				  $this->sqlCheck = $SQL;
			      $r = mysql_query($SQL, $this->conexao);
				  if ($r === false){
					 $this->lastError = mysql_error();
				    $result = false;
				  }else{
                    $this->Commit();
					$result = $ra;
				  }
				  break;
			case "PACK" :
				  $result = $this->_PackTable();
				  break;
			case "EMPTY" :
				  $ra = $this->_GerRowsAfec();
				  $SQL  = "TRUNCATE $this->table";
				  $this->sqlCheck = $SQL;
			      $r = mysql_query($SQL, $this->conexao);
				  if ($r === false){
					 $this->lastError = mysql_error();
				    $result = false;
				  }else{
					 $result = $ra;
				  }
				  break;
			 default :
			     $this->lastError = "Comando no suportado '$this->command'";
			     $result = false;
		   }
	      return $result;
	   }
	}

  } //r: Classe
?>
