<?php
  /* ---------------------------------------------------------------------------
     uLib Versao 3.0
	 Autor: Alexandre P. Trindade
     Copyright 2004 - Uniware Acessoria e Consultoria de Sistemas LTDA.
     Inicio: 15/09/2004
	 ---------------------------------------------------------------------------
  */

  uses("objects","lib_misc","jc_padrao");

  //r: converte um array em uma string com quebra de linhas
  function array_break_line($arr){
    $result = "";
    if (is_array($arr)){
       foreach($arr as $k => $v){
          $result .= "$v\r\n";
       }
    }
    return $result;
  }

  /**
   * uWSServer
   * @author   Alexandre P. Trindade <alexpt@uniware.com.br>
   * @version  o_rclass.php ver. 1.0  28/01/2009
   * @access   public
   *
   * uWSServer implementa protocolo de comunicao UWS1 (Uniware Web Service verso 1.0)
   * Lado servidor
   *
   * Tipos de dados reconhecidos:
   * str   - String
   * int   - Inteiro
   * real  - Real
   * bool  - Boleano
   * date  - Data
   * time  - Hora
   * timestamb - Data e hora
   * bin   - binrio, qualquer tipo de informao no suportado pelos tipos anteriores,
   *         como imagens, arquivos, etc.
   * complex - estrutura definida pelo programador
   *
   * Padro para definio de estruturas, tipos complexos (cada elemento de $a_structs)
   *
   *  tipo = array("name" => "nomeDoTipo",            ** Nome que identifica este tipo
   *               "type" => "struct/array",          ** Tipo de saida (struct -> array[x], array -> array[x,y])
   *
   *               "struct" => array(                 ** Composio do tipo se struct (ou elemento de array)
   *                                  "nomeCampo1" => "str",
   *                                  "nomeCampo2" => "int",
   *                                  "nomeCampo3" => "date"
   *                                ),
   *               "struct" => "nomeDoTipo"           ** Composio do tipo se array, aponta para o nome de tipo que define cada elemento (linha anterior)
   *              )
   *
   *
   * Padro para descrio de mtodos (cada elemento de $a_metodos):
   *   Metodo = array( "name" => "nomeMetodo",                  ** Nome puiblico do metodo e nome da funao PHP correspondente
   *                  "input" => array("paramName1" => "str",  ** Nome e tipo do primeiro pametro de entrada
   *                                   "paramName2" =  "int",  ** nome e tipo do segundo, e assim por diante
   *                                   ),
   *                  "outType" => "single/complex/void",      ** layout de saida (single -> Retorna um tipo base simples, complex - retorna um tipo definido no array$a_structs)
   *                  "output"  => "int/nomeDoTipo",           ** tipo retornado
   *                 )
   *
   */
  class uWSServer extends uObject {

    var $a_metodos;  //r: Array contendo a lista de metodos publicados
    var $a_structs;  //r: Array de tipos/estruturas utilizadas
    var $method;     //r: Nome do metodo invocado
    var $fault_flag;
    var $fault_error;

    function uWSServer() {
	     $name      = (func_num_args() > 0)?  func_get_arg(0):"nonamed-uWSServer";
	     $classPath = (func_num_args() > 1)?  "uWSServer.".func_get_arg(1):"uWSServer";

		 $this->uObject($name,$classPath); //r: inherited

		 $this->a_metodos = array();
		 $this->a_structs = array();
		 $this->fault_flag = false;   //r: Flag de erro se true retorna um registro padro de erro
		 $this->fault_error = "";     //r: Mensagem a retornar;
		 $this->method = "";
    }

	/*
	 * registerType  - Registra os tipos de dados complexos, que so retornados
	 *                 pelos mtodos publicados
	 * @access	public
	 *
	 * Parametros:
	 * @param String $name Nome que identifica este tipo (nome de tipo deve ser unico para cada tipo)
	 * @param String $type Tipo de resultado  "struc" = Estrutura simples (array mono dimencional)
    *                     "array" = Array de "struct" (array bidimencional)
    * @param Mix $struc Array definido a estrutura (se type = "struct")
    *                   ex. (1): array("nomeCampo1" => "str","nomeCampo2" => "int") )
    *                   ou uma string contendo o nome da estrutura que define os elementos do array
    *                   se $type == "array".
    *                   ex. (2):  "funcaoXResult"
	*/

    function registerType($name,$type,$struc){
       $novo = array( "name" => $name,
                      "type" => $type,
                      "struct" => $struc);
       array_push( $this->a_structs, $novo);
    }

    /* registerMethodo - Registra os mtodos disponiveis na interface do servio
     *
     * @access public
     *
     * Parmetros:
     * @param String $name Nome do mtodo na interface e da funo PHP correpondente
     * @param Array  $input Array contendo a lista de parmetros de entrada do metodo
     *                      p.ex: array("paramName1" => "str","paramName2" => "int").
     * @param String $outType String identificando o tipo de retorno do metodo
     *                        (single/complex/void -> simples/complexo/sem retorno)
     * @param String $output String definindo o tipo retornado ou o nome de um tipo complexo
     *
     *
    */
    function registerMethodo($name,$input=array(),$outType="single",$output="int"){
       $novo = array("name" => $name,
                     "input" => $input,
                     "outType" => $outType,
                     "output" => $output );
       array_push($this->a_metodos, $novo);
    }

    /*
     * Seta erro para retorno
     *
     * @access public
     * @param String $msg Mensagem de erro a retornar para a aplicao cliente
    */
    function fault($msg){
       $this->fault_flag = true;
       $this->fault_error = $msg;
    }

    //r: retorna o valor padro para o tipo de dado
    function emptyData($dataType){
       $result = "";
       switch ($dataType){
         case "int" :
         case "real" :
               $result = 0;
               break;
         case "bool" :
               $result = 0;
               break;
         case "time" :
               $result = "00:00:00";
               break;
         case "date" :
               $result = "1899-12-31";
               break;
         case "timestamp" :
               $result = "00000000000000";
       }
       return $result;
    }

    /*
     * Retorna a lista contendo os nomes dos parmetros de entrada
     *
     * @acces private
    */
    function getImputParamsList( $metodName ){
      $result = array();
      $i = 0; $achou = false;
      while (($i < count($this->a_metodos)) && !$achou){
         $e = $this->a_metodos[$i];
         if ($e["name"] == $metodName){
            $achou = true;
            $a = $e["input"];
            if (is_array($a)){
               //r: obtem a lista dos nomes de parmetros
               foreach($a as $k => $v){
                 // $result[] = array($k => $this->emptyData($v) );
                 $result[$k] = $this->emptyData($v);
               }
            }else{
              $result = $a;
            }
         }
         $i += 1;
      }
      return $result;
    }


    /*
     *  Retorna a descrio de um tipo
     *
     */
    function getType( $typeName ){
      $result = array("header" => "","dataType" => $typeName, "struct" => array() );
      $dataType = $typeName;
      $i = 0; $achou = false;
      while ($i < count($this->a_structs) && !$achou){
        $tipo = $this->a_structs[$i];
        if ($tipo["name"] == $typeName){
           $ax = $tipo["struct"];
           if ( is_array( $ax ) ){   //r: se struct aponta para um array  a descrio da estrutura
              $result["header"] = (empty($result["header"]))?  "struct":$result["header"];
              $result["dataType"] = $dataType;
              $result["struct"] = $ax;
              $achou = true;
           }else{                   //r: se no  o nome do elemento que contem a descrio
             $result["header"] = (empty($result["header"]))?  "array":$result["header"];
             $typeName = $ax;       //r: seta o novo nome de tipo que deve ser procurado
             $dataType .= ".$ax";
             $i = -1;               //r: reinicia a busca para o novo nome
           }
        }
        $i += 1;
      }
      if ($achou){
         return $result;
      }else{
         return false;
      }
    }

    /*
     *  Retorna a estrutura de saida do mtodo
     *
     */
    function getMethodType(){
      $result = false;
      $i = 0; $achou = false;
      while (($i < count($this->a_metodos)) && !$achou){
         $e = $this->a_metodos[$i];
         if ($e["name"] == $this->method){
            $achou = true;
            $a = $e["outType"];
            if ($a == "void"){
                $result = array("header" => "void","dataType" => "void");
            }else if ($a == "single"){
                $result = array("header" => "single","dataType" => $e["output"]);
            }else{
                $result = $this->getType( $e["output"] );
            }
         }
         $i += 1;
      }
      return $result;
    }

    /*
     * Retorna erro
     *
     * @acces private
     */
    function getFault(){
       $result = array();
       $result[] = "fault|0|0";
       $result[] = $this->fault_error;
       return $result;
    }

    /*
     * Monta linha de dados de resultado
     *
     */
    function mountDataLine( $arr ){
      $result = "";
      if (is_array( $arr )){
          foreach($arr as $k => $v){
            $result .= (empty($result))? "":"|";
            if ($v == "0"){
               $result .= "36";  //r: Hex_encode retorna vazio para o valor Zero
            }else{
               $result .= hex_encode( (string) $v );
            }
          }
      }
      return $result;
    }

    /*
     * getDelta - Monta retorno conforme a interface especificada
     *
     * @acces private
     *
     */

    function getDelta($var){
       $result = array();
       $struc = $this->getMethodType();

       if ($struct === false){
          //r: erro na descrio do resultado do metodo
          $this->fault("UWS Erro: Tipo de retorno no definido para o metodo " . $this->method . "().");
          $result = $this->getFault();
       }else{
          //r: monta retorno conforme  registro do metodo
          if ($struc["header"] == "void"){
              //r: o metodo no retorna nenhum valor (procedure)
              $result[] = hex_encode( "void|0|0" );               //r: Header
          }else if ($struc["header"] == "single"){
              //r: o mtodo retorna um tipo simples
              $result[] = hex_encode( $struc["header"]."|1|1" );  //r: Header
              $result[] = hex_encode("|" . $struc["dataType"]);   //r: Descritor
              if ($var == "0"){
                 $result[] = "36";
              }else{
                 $result[] = hex_encode( (string) $var );            //r: Linha de dados (Valor de retorno)
              }
          }else{
              //
              //r: retorna struct ou array
              //

              //r: reserva a linha do header
              $result[] = $struc["dataType"];

              $rc = 0; $fc = 0;

              //r: monta o descritor dos campos
              $ax = $struc["struct"];

              foreach($ax as $k => $v){
                 $result[] = hex_encode((string) "$k|$v");
                 $fc += 1;
              }

              //r: anota o numero de campos (linhas no descritor)
              $result[0] .= "|$fc";


              if (is_array($var)){

                  //r: monta a rea de dados
                  if ($struc["header"] == "struct"){
                     //r: apenas 1 registro com 1 ou mais campos
                     $result[0] .= "|1";
                     //r: monta linha de dados
                     $result[] = $this->mountDataLine( $var );
                  }else if ($struc["header"] == "array"){
                     //r: 1 ou mais registros
                     $result[0] .= "|" . count($var);
                     //r: monta linhas de dados
                     foreach($var as $k => $v){
                       $result[] = $this->mountDataLine( $v );
                     }
                  }
                  $result[0] = hex_encode($result[0]);
             }else{
                $this->fault("UWS Erro: Metodo " . $this->method . "(). o valor retornado no  um array.");
                $result = $this->getFault();
             }
          }
       }
       return $result;
    } //r: fim de getDelta()


    /*
     *  Posta o resultado
     *
     * @acces private
     */
    function postResult($var){
       $buff = "";
       if ($this->fault_flag){
          $buff = $this->getFault();
       }else{
          $buff = $this->getDelta($var);
       }
       echo "\r\n<!--UWS\r\n", array_break_line($buff) ,"UWS-->\r\n";
    }


    /*
     *  service - Ativa o servio
     *
    */
    function service(){
        $metodName = hex_decode( RParams::get_t("methodName","") );
        $result = "@NULL_METHOD_RESULT@";
        $command = "";
        if (!empty($metodName)){
           $this->method = $metodName;
           $paramList = $this->getImputParamsList($metodName);
           if (count($paramList) > 0){
              // l a lista de parmetros recebidos
              $pl = ""; $i = 0;
              $a_params = array();
              foreach($paramList as $k => $v){
                $valRead = RParams::get_t($k, $v);
                $a_params[] = hex_decode( $valRead );
                $ax = '$a_params['.$i.']'; $i++;
                $pl.= (empty($pl))? $ax:",$ax";

              }
              $command  = '$result = ';
              $command .= "$metodName(".$pl.");";
           }else{
              //r: no tem parametro de entrada
              $command = '$result'." = $metodName();";
           }
           eval($command);
        }else{
           $this->fault("UWS erro: Metodo no informado.");
        }

        //r: posta o resultado do metodo
        $this->postResult($result);
    }

  }
?>
