<?php
  /*  Classe para manipulao de arquivos XML
   *  Programador: Alexandre P. Trindade
   *  Data.: 21/10/2010
   *
   *  Classe uXMLItem - Cada Tag do arquivo XML
   *  Classe uXML     - Analizador XML (L/Manipula/Grava)
   *                    Com Suporta a CDATA
   */

   //r: CharSets Suportados: ISO-8859-1, UTF_8
   $XMLINDENTAR    = false;
   $XMLCHARSETIN   = "UTF_8";
   $XMLCHARSETOUT  = "ISO-8859-1";

   function concat(){
      $ac = func_num_args();
      $result = "";
      for($i = 0; $i < $ac; ++$i){
        $result .= func_get_arg($i);
      }
      return $result;
   }

   //r: LoadCharSet
   function lcs($value){
     global $XMLCHARSETIN;

     if ($XMLCHARSETIN == "UTF_8"){
        return utf8_decode($value);
     }else{
        return $value;
     }
   }

   //r: SaveCharSet
   function scs($value){
     global $XMLCHARSETOUT;

     if ($XMLCHARSETOUT == "UTF_8"){
        return utf8_encode($value);
     }else{
        return $value;
     }
   }

   function show($msg){
     echo $msg,"\r\n";
   }

   /**
    *  uXMLItem represanta cadxa uma das tags do arquivo XML individualmnente.
    *
    *  NOTA: Para o carregamento e parse de XML seja um arquivo ou uma string
    *         obrigatrio que o formato de entrada seja UTF_8. o parser do
    *        PHP no funciona com outro CharSet. Portanto quando o XML alvo
    *        estiver em um padro diferente, usar utf_encode() para carregar
    *        a stringo do XML.
    */
   class uXMLItem{
      /**
       * XML container principal
       * @var uXML
       */
      var $mainXML;
      /**
       * Nvel do elemento (Node/Campo)
       * @var Int
       */
      var $level;
      /**
       * Nome do elemento (Tag)
       * @var String
       */
      var $name;
      /**
       * Valor do elemento (Tag)
       * @var Mix
       */
      var $value;
      /**
       * ndice do elemento no array childs do elemento que o contem
       * @var Int
       */
      var $index;
      /**
       * Flag de ativao de conteudo protegido para o valor do elemento
       * Se true, envolve o contedo numa tag <![CDATA[]]>
       * @var type
       */
      var $cdata;
      /**
       * Lista de atributos da tag (array("atributo"=>valor))
       * @access Private
       * @var type
       */
      var $attribs;
      /**
       * Array contendo os elementos filhos
       * @var uXMLItem array(uXMLItem,uXMLItem,...)
       */
      var $childs;
      /**
       * Elemento (tag) pai
       * @var uXMLItem
       */
      var $parent;

      /**
       * uXMLItem Creator
       */
      function uXMLItem(){
          $this->mainXML = NULL;
          $this->level   = 0;
          $this->name    = "";
          $this->value   = "";
          $this->index   = -1;
          $this->cdata   = false;
          $this->attribs = array();
          $this->childs  = array();
          $this->parent = NULL;
      }

      function say($msg){
         $ident = str_repeat("   ",$this->level);
         echo $ident, $msg, "\r\n";
      }

      /**
       * Excuta um dum correspondnte ao elemento (tag) XML selecionado.
       */
      function dumpXML(){
         $tab = "  ";
         $cdata = ($this->cdata)? "true":"false";
         $this->say("TagName: ".$this->name);
         $this->say("  level: ".$this->level);
         $this->say("  cdata: $cdata");
         $this->say("  value: ".$this->value);
         $this->say("Attribs: count=".count($this->attribs));
         if (count($this->attribs) > 0){
            reset($this->attribs);
            foreach($this->attribs as $k => $v){
              $this->say("  $k = \"$v\"");
            }
            reset($this->attribs);
         }
         $this->say("---------------------------------------");
         if (count($this->childs)>0){
            reset($this->childs);
            foreach($this->childs as $child){
               $child->dumpXML();
            }
            reset($this->childs);
         }
      }

      function is_xmlItem(){
         return true;
      }

      /**
       * Retorna a quantidade e elementos filhos que o elemento (tag) XML
       * selecionado contem.
       *
       * @return Int
       */
      function count(){
         return count($this->childs);
      }

      /**
       * Seta o valor para um atributo do elemento (tag) XML selecionado.
       * Os atributos podem ser identificado por quaisquer nomes desde que
       * no contenham espaos ou caracteres especiais
       *
       * Atributos fixos:
       * name  - Nome da tag
       * value - valor da tag
       * cdata - true/false ativa/desativa CDATA para valor da tag
       *
       * Atributos reservados (uso interno noo setar estes):
       * level, index
       *
       * @param String $attrName Nome do atributo
       * @param type $attrValue Valor do atributo
       * @return uXMLItem Retorna o elemento XML selecionado
       */
      function set_attr($attrName,$attrValue){
          if ($attrName == "level"){
             $this->level = $attrValue;
          }else if ($attrName == "name"){
             $this->name = $attrValue;
          }else if ($attrName == "value"){
             $this->value = $attrValue;
          }else if ($attrName == "index"){
             $this->index = $attrValue;
          }else if ($attrName == "cdata"){
             $this->cdata = $attrValue;
          }else{
            $this->attribs[$attrName] = $attrValue;
          }
          return $this;
      }

      /**
       * Retorna o valor de um atributo do elemento XML selecionado
       *
       * @param String $attrName Nome do atribudo
       * @return Mix Valor correspondente ao atributo solicitado ou vazio.
       */
      function get_attr($attrName){
          return $this->attribs[$attrName];
      }

      /**
       * Retorna o objeto correspondente ao elemento (tag) XML informado.
       *
       * @param String $childName Nome do elemento (tag) XML cujo objeto dever
       *                          ser retornado.
       * @return uXmlItem Retorna o objeto do elemento solicitado ou false em
       *                  caso de falha.
       */
      function itemByName($childName){
         $i = 0; $result = false;
         while ($i < count($this->childs) && $result===false){
            if (strtolower($childName) == strtolower($this->childs[$i]->name)){
               $result = $this->childs[$i];
            }
            $i+=1;
         }
         return $result;
      }

      /**
       * Retorna o valor de um Elemento (tag) XML contido neste elemento ou
       * $default se o elemento indicado no existir na lista de items.
       *
       * @param String $childName Nome do elemento cujo valor ser retornado.
       * @param Mix $default Valor a ser retornado se o elemento solicitado no
       *                     existir.
       * @return Mix Valor do elemento (tag) solicitado ou o valor informado no
       *             parmetro $default.
       */
      function itemValue($childName,$default=""){
         $result = $this->itemByName($childName);
         if ($result !== false){
            return $result->value;
         }else{
            return $default;
         }
      }

      /**
       * Adiciona um novo elemento XML e retoprna o ndice para o elemento
       * @return Int
       */
      function addChild(){
         $result = count($this->childs);
         $this->childs[] = new uXMLItem();
         $this->childs[$result]->level   = $this->level+1;
         $this->childs[$result]->mainXML = $this->mainXML;
         $this->childs[$result]->parent  = $this;
         return $result;
      }

      /**
       * Adiciona um novo elemento XML e retorna um ponteiro para o
       * elemento adicionado.
       *
       * @param String $aName Nome para o novo elemento (tag).
       *
       * @return uXMLItem
       */
      function newItem($aName){
         $r = $this->addChild();
         $this->childs[$r]->name = $aName;
         return $this->childs[$r];
      }

      /**
       * Exclui o elemento (tag) XML da posio indicada.
       * @param Int $index ndice do elemento (tag) XML a ser excluido.
       */
      function deleteChild($index){
          unset($this->childs[$index]);
      }

      /**
       * Exclui um atributo de um elemento (tag) XML.
       * @param String $attrName Nome do atributo a ser excluido
       */
      function deleteAttrib($attrName){
         unset($this->attribs[$attrName]);
      }

      /**
       * @global boolean $XMLINDENTAR
       * @return String Retorna uma texto XML correspondente a estrutura do elemento XML
       */
      function save(){
         global $XMLINDENTAR;

         $identacao = "";
         if ($XMLINDENTAR){
            $identacao = str_repeat("   ",$this->level);
         };
         $endIdent = $identacao;

         $result = concat("$identacao<",$this->name);
         $childCount =  count($this->childs);
         $attrCount  =  count($this->attribs);
         $valor      =  $this->value;

         //r: grava os attribs
         if (count($this->attribs) > 0){
            reset($this->attribs);
            foreach($this->attribs as $k => $v){
               $result .= concat(" ",$k,"=","\"$v\"");
            }
         }
         $cdataIni = "";
         $cdataEnd = "";
         if ($this->cdata){
            $cdataIni = '<![CDATA[';
            $cdataEnd = ']]>';
         }
         // se  um node no possui valor de campo
         if ($childCount > 0){
            $this->value = "";
         }else{
            $endIdent = "";
         }

         //r: verifica se a tag nao esta vazia
         if (empty($this->value) && count($this->childs)==0){
            if ($this->mainXML->singleEndTag){
                $result .= " />\r\n";
            }else{
              $result .= concat(">\r\n","</",$this->name,">\r\n");
            }
         }else{
            $result .= '>';
            if ($childCount > 0){
               $result .= "\r\n";
            }

            //r: no esta vazio, salvar valor ou cada um dos itens (child)
            if (!empty($this->value)){
               $result .= $cdataIni . scs($this->value) . $cdataEnd;
            }else{
               reset($this->childs);
               foreach($this->childs as $child){
                 $result .= $child->save();
               }
            }
            $result .= concat("$endIdent</",$this->name,">\r\n");
         }

         return $result;
      }
   }

   /**
    *  Manipulador XML
    */
   class uXML{
     /**
      * Documento XML
      * @var uXMLItem;
      */
     var $xmlDoc;
     /**
      * fechamento de tag simples para campos vazios (p.ex: true = <nome />, false = <nome></nome>)
      * @var Boolean
      */
     var $singleEndTag;  //r:

     /**
      * uXML Creator
      */
     function uXML(){
        global $XMLINDENTAR;
        global $XMLCHARSETIN;
        global $XMLCHARSETOUT;

        $XMLINDENTAR    = false;
        $XMLCHARSETIN   = "UTF_8";
        $XMLCHARSETOUT  = "ISO-8859-1";

        $this->singleEndTag = true;
        $this->xmlDoc = new uXMLItem();
        $this->xmlDoc->mainXML = $this;
        $this->xmlDoc->level   = 0;
        $this->xmlDoc->index   = 0;
        $this->xmlDoc->name    = "uXMLDoc";

     }

     /**
      *
      * @global String $XMLCHARSETOUT
      * @param String $newCharset Charset padro para o  arquivo XML gerado
      *                           "ISO-8859-1" ou "UTF_8".
      */
     function charsetOut($newCharset="ISO-8859-1"){
        global $XMLCHARSETOUT;
        $XMLCHARSETOUT = $newCharset;
     }

     /**
      * Seta o Charset a ser utilizado para o XML a ser carregado do arquivo ou string
      *
      * @global type $XMLCHARSETIN
      *
      * @param String $newCharset Charset padro para o  arquivo XML a ler
      *                           "ISO-8859-1" ou "UTF_8".
      */
     function charSetIn($newCharset="UTF_8"){
        global $XMLCHARSETIN;
        $XMLCHARSETIN = $newCharset;
     }

     /**
      * Salvar o XML atual no arquivo
      * @global String $XMLCHARSETOUT
      * @param String $aFileName Nome e path do arquivo a ser gravado
      * @return Boolean True/False para Sucesso ou falha
      */
     function saveToFile($aFileName){
        global $XMLCHARSETOUT;

        $buff = "<?xml version='1.0' encoding='$XMLCHARSETOUT'?>\r\n";
        $buff .= $this->xmlDoc->save();

        $handle = fopen($aFileName,"w+");
        if ($handle !== false){
           $size = strlen($buff);
           $writen = fwrite($handle,$buff,$size);
           fclose($handle);
           return ($writen > 0);
        }else{
           return false;
        }
     }

     /**
      * Retorna uma string contendo o XML atual no formato texto.
      * @global String $XMLCHARSETOUT
      * @return String
      */
     function getXML(){
        global $XMLCHARSETOUT;

        $buff = "<?xml version='1.0' encoding='$XMLCHARSETOUT'?>\r\n";
        $buff .= $this->xmlDoc->save();
        return  $buff;
     }

     /**
      * Cria um novo documento XML. Qualquer conteudo previo existente ser
      * perdido.
      *
      * @param String $nameDoc Nome da gat principal do arquivo XML
      *
      */
     function newDoc($nameDoc = "uXMLDoc"){
        unset($this->xmlDoc);
        $this->xmlDoc = new uXMLItem();
        $this->xmlDoc->name = $nameDoc;
     }

     function DOMType($xmlDOM,$xmlItem,$setCDATA=true){
        $result = 0; // r: sem tipo definido
        $nodeName  = $xmlDOM->nodeName;
        $s = trim($xmlDOM->nodeValue);
        $fc = substr($nodeName,0,1);
        $fv = substr($s,0,1);
        $p1 = ($fv == chr(10));
        $p2 = ($fc == '#');
        if ( !$p1 && !$p2){
           $result = 1;  //r: Node com valor definido
        }else if ($nodeName == "#cdata-section"){
           $result = 3;  //r: tag CDATA;
           if ($setCDATA){
              $xmlItem->cdata = true;
              $xmlItem->value = lcs($xmlDOM->nodeValue);
           }
        }else if (!$p2){
           $result = 2;  //r: Node sem valor definido
        }

        return $result;
     }

     function loadDOM($xmlDOM,$xmlItem){
        $nodeName  = $xmlDOM->nodeName;
        $type = $this->DOMType($xmlDOM,$xmlItem,false);
        if ($type == 1){
           $nodeValue = $xmlDOM->nodeValue;
        }else{
           $nodeValue = "";
        }

        $localName      = $xmlDOM->localName;
        $xmlItem->name  = $xmlDOM->nodeName;
        $xmlItem->value = lcs($nodeValue);

        // r: carrega atributos se houver algum
        $ac = $xmlDOM->attributes->length;
        if (!empty($ac) && $ac > 0){
           for ($i=0; $i < $ac; ++$i){
               $anome  = $xmlDOM->attributes->item($i)->name;
               $avalue = $xmlDOM->attributes->item($i)->value;

               $xmlItem->attribs[$anome] = lcs($avalue);
           }
        }

        // r: Carrega childNodes se houver
        $t = $xmlDOM->childNodes;
        $i = 0;

        while ($i < $t->length){ // -1
           $childDOM = $t->item($i);
           $tipo = $this->DOMType($childDOM,$xmlItem, true);

           if ($tipo == 1 || $tipo == 2){
               $tmp = $xmlItem->addChild();
               $item = $xmlItem->childs[$tmp];
               $this->loadDOM($childDOM, $item);
           }
           $i += 1;
        }
     }

     /**
      * Carrega XML de um arquivo em disco.
      * @param string $xmlFileName Nome e caminho do arquivo a ser lido
      * @return Boolean True/False  para sucesso ou falha.
      */
     function loadFromFile($xmlFileName){
        if (file_exists($xmlFileName)){
            $this->newDoc();
            $xml = new XMLReader();
            if ($xml->open($xmlFileName)){
                $lido = false;
                while (!$lido && $xml->read()){
                   if ($xml->nodeType == XMLReader::ELEMENT){
                      $lido = true;
                      $nodeEx = $xml->expand();
                      $nodeEx->normalize();
                      $this->loadDOM($nodeEx, $this->xmlDoc);
                   }
                }
                return true;
            }else{
               return false;
            }
        }else{
           return false;
        }
     }

     /**
      * Carrega XML de uma string
      * @param String $XMLString String contendo XML no formato texto.
      * @return Boolean True/False  para sucesso ou falha.
      */
     function loadFromString($XMLString){
        if (!empty($XMLString)){
            $this->newDoc();
            $xml = new XMLReader();
            if ($xml->xml($XMLString)){
                $lido = false;
                while (!$lido && $xml->read()){
                   if ($xml->nodeType == XMLReader::ELEMENT){
                      $lido = true;
                      $nodeEx = $xml->expand();
                      $nodeEx->normalize();
                      $this->loadDOM($nodeEx, $this->xmlDoc);
                   }
                }
            }else{
            }
            return true;
        }else{
           return false;
        }
     }


   }
?>
