<?php

  /* ---------------------------------------------------------------------------
     uLib Versao 3.0
	 Autor: Alexandre P. Trindade
     Copyright 2004 - Uniware Acessoria e Consultoria de Sistemas LTDA.
     Inicio: 27/02/2009
	 ---------------------------------------------------------------------------
  */

  define(DIRECTORY_SEPARATOR, '/');
  
  class uZip {
     var $source;        //r: Pasta de origem para compactao de arquivos
     var $dest;          //r: Pasta de destino para descompactao de arquivos
     var $includeFiles;  //r: Lista de arquivos a serem compactados/descompactados (array)
                         //r: ou mascara (string) ('*.*','*.exe',etc)
     var $excludeFiles;  //r: Lista de arquivos ou mascara para excluso (ignorados pelo processo compactacao ou descompactao)
     var $lastError;

     function uZip(){
       $this->source = "";
       $this->dest = "";
       $this->includeFiles = "*.*";
       $this->excludeFiles = "";
       $this->lastError = "";
     }

     /* testaNome()  - Verifica se um nome bate com a mascara indicada
      *
      * @access private
      *
      * @parameter fileName: Nome do arquivo para checagem
      * @parameter mask:     Mascara para validao
      *
      * @return  True se o arquivo satisfizer o contido na mascara
      *
      */

     function testaNome($fileName,$mask){
         $result = false;
         $aux = $mask;
         if ($aux == "*.*"){
            $result = true;
         }else if ( preg_match( '#^\*\.#', $aux ) ){

            //r: qualquer arquivo com a extenso indicada
            $pi = pathinfo( "/$aux" );
            $aux = '#\.' . $pi["extension"] .'$#';
            $result = preg_match( $aux, $fileName );

         }else if ( preg_match( '#\.\*$#', $aux ) ){

            //r: Nome e path especificado qualquer extensao
            $pi = pathinfo( $aux );
            $aux = '#^' . $pi["dirname"]."/".$pi["basename"].'#';
            $result = preg_match( $aux, $fileName );


         }else{

            //r: Nome e path especfico
            $result = preg_match( '#^'. $aux .'$#', $fileName );

         }
         
         return $result;
     }

     /* estaNaLista()  - Verifica se um nome de arquivo esta na lista para
      *                  compactao ou descompactao
      * @access private
      *
      * @parameter fileName: Nome do arquivo para checagem
      *
      * @return  True se o arquivo estiver na lista.
      *
      */
     function estaNaLista( $fileName ){
        $result = false;
        if ( !is_array( $this->includeFiles ) ) {
           $result = $this->testaNome( $fileName, $this->includeFiles);
        }else{
           $i = 0;
           while (!$result && $i< count($this->includeFiles)){
              $result = $this->testaNome($fileName, $this->includeFiles[$i]);
              $i += 1;
           }
        }
        
        return $result;
     }
     
     
     /* extractFrom()  - Extrai arquivos apartir de um arquivo ZIP
      *
      * @access public
      *
      * @parameter zipfileName: Nome do arquivo zip do qual sero extraidos os arquivos
      *
      * @return  True se o arquivo satisfizer o contido na mascara
      *
      */
     function extractFrom($zipFileName){
       $result = false;
       $testeFile = "";
       $this->lastError = "";

       $dirDest = $this->dest;

       $zip = zip_open($zipFileName);
       if ($zip){
          $result = true;
          while ($zip_entry = zip_read($zip)){
            //r: Monta o camino para a pasta de destino para o arquivo corrente
            $completePath = $dirDest . '/' . dirname(zip_entry_name($zip_entry));
            //r: Monta o path compledo de destino do arquivo
            $zName = zip_entry_name($zip_entry);
            $completeName = $dirDest . '/' . $zName;

            //r: Verifica se o arquivo corrente deve ser descompactado
            $ok = $this->estaNaLista( $zName );
            
            //r: Guarda um nome de arquivo para teste de extracao
            if ($ok) {
                $testeFile = $completeName;
            }
            
            //r: Se vai descompactar, checa se a pasta de destino existe,
            //r: se no existir cria
            if ($ok && !file_exists($completePath)){
               //r: Criar pasta de destino
               $tmp = '';
               foreach(explode('/',$completePath) AS $k){
                  $tmp .= $k.'/';
                  if (!file_exists($tmp) ){
                      @mkdir($tmp, 0777);
                  }
               }
            }
            
            //r: salva o arquivo descompactado
            if ($ok){
               if (zip_entry_open($zip, $zip_entry, "r")){
                  if ($fd = @fopen($completeName, 'w+')){
                     //r:  um arquivo, grava o arquivo descompactado
                     fwrite($fd, zip_entry_read($zip_entry, zip_entry_filesize($zip_entry)));
                     fclose($fd);
                  }else{
                     //r:  uma pasta vazia, cria a pasta
                     @mkdir($completeName, 0777);
                  }
                  zip_entry_close($zip_entry);
               }
            }
            
          }
          
          zip_close($zip);
          
          if ($result){
             // verifica se o ultimo dos arquivos compactados
             $result = file_exists($testeFile);
          }
       }else{
         if ( !file_exists($completePath) ){
            $this->lastError = "Arquivo no encontrado $zipFileName.";
         }else{
            $this->lastError = "Erro ao abrir o arquivo $zipFileName.";
         }
         $result = false;
       }
       return $result;
     }

  }
?>
